// AddInstanceDir.cpp : implementation file
//

#include "stdafx.h"
#include "ControlPanel.h"
#include "AddInstanceDir.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CAddInstanceDir property page

IMPLEMENT_DYNCREATE(CAddInstanceDir, CWizardPage)

CAddInstanceDir::CAddInstanceDir() : CWizardPage(CAddInstanceDir::IDD)
{
	//{{AFX_DATA_INIT(CAddInstanceDir)
	m_Path = _T("");
	//}}AFX_DATA_INIT
}

CAddInstanceDir::~CAddInstanceDir()
{
}

void CAddInstanceDir::DoDataExchange(CDataExchange* pDX)
{
	CPropertyPage::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CAddInstanceDir)
	DDX_Control(pDX, IDC_BROWSE, m_Browse);
	DDX_Control(pDX, IDC_PATH, m_EditPath);
	DDX_Text(pDX, IDC_PATH, m_Path);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CAddInstanceDir, CPropertyPage)
	//{{AFX_MSG_MAP(CAddInstanceDir)
	ON_BN_CLICKED(IDC_BROWSE, OnBrowse)
	ON_WM_PAINT()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CAddInstanceDir message handlers


void CAddInstanceDir::OnBrowse() 
{
	// TODO: Add a "New Folder" button to this dlg

	char path[MAX_PATH];
	CString title((LPCSTR) IDS_PROMPT_BROWSE_INSTANCE);

	BROWSEINFO bi;
	ZeroMemory(&bi, sizeof(bi));
	bi.hwndOwner = m_hWnd;
	bi.lpszTitle = title;

	m_EditPath.GetWindowText(path, MAX_PATH);

	ITEMIDLIST *idList;
	idList = SHBrowseForFolder(&bi);
	if (idList)
	{

        if ( SHGetPathFromIDList ( idList, path ) )
        {
			m_EditPath.SetWindowText(path);
        }


		// Free mem
		IMalloc *imalloc = 0;
		if ( SUCCEEDED( SHGetMalloc ( &imalloc )) )
		{
			imalloc->Free(idList);
			imalloc->Release();
		}
	}

	
}

BOOL CAddInstanceDir::OnSetActive() 
{

	if ((SourceType) QuerySiblings(0, 0) == AttachExisting)
	{
		GetDlgItem(IDC_MSG)->SetWindowText("You have chosen to create a configuration to control an existing Zope installation.  Please specify the directory which contains the instance of this installation.");
	}
	else
	{
		GetDlgItem(IDC_MSG)->SetWindowText("Each Zope configuration must have its own instance directory.  This is where Zope stores its data and log files.  You can accept the suggested default value or enter your own below.");
		if (!m_EditPath.GetModify())
		{
			m_Path = CZopeDialog::AddTrailingSlash(zopeDialog->m_DefaultInstanceDir) + (LPCSTR) QuerySiblings(0, 1);
			UpdateData(FALSE);
			m_EditPath.SetModify(FALSE);
			
		}
	}

	
	return CPropertyPage::OnSetActive();	

}

void CAddInstanceDir::OnPaint() 
{
	CPaintDC dc(this); // device context for painting
	
	Paint(dc);	
}



LRESULT CAddInstanceDir::OnWizardNext() 

{
	SourceType source = (SourceType) QuerySiblings(0, 0);

	UpdateData(TRUE);
	// Check if path exists
	CFileStatus status;
	// We have to REMOVE the trailing slash in order
	// for GetStatus to find it
	if (m_Path.Right(1) == "\\")
		m_Path = m_Path.Left(m_Path.GetLength() - 1);

	if (CFile::GetStatus(m_Path, status))
	{
		// Check if ZOPE is installed in this dir
		CString testPath = m_Path;
		testPath += "\\";
		testPath += "var";

		CString msg;
		if (CFile::GetStatus(testPath, status))
		{
			// Zope does exist
			if (source != AttachExisting)
				msg = "The directory you have selected already exists and appears to contain a Zope instance within it.  If you continue with this choice some or all of the data from this instance will be lost.  Are you sure you want to proceed?";
			else
				return IDD_ADD_FINISH;
		}
		else
		{
			// Dir exists but does not contain Zope
			if (source != AttachExisting)
				msg = "The directory you have selected already exists.  There is a chance that you may lose some data if your proceed with this choice.  Are you sure you want to continue?";
			else
				msg = "The directory you have selected does not appear to contain a valid Zope instance.  Zope instances must at the very least contain a 'var' directory.  Do you want to proceeed with your selection anyway?";
		}

		if (MessageBox(msg, theApp.m_DialogCaption, MB_ICONEXCLAMATION | MB_YESNO) == IDNO)
			return -1;
	}
	else if (source == AttachExisting)
	{
		// Directory does not exist
		MessageBox("The instance directory you have specified does not exist.  Please enter another.", theApp.m_DialogCaption, MB_ICONEXCLAMATION);
		return -1;
	}
	
	// For setup wizard, this is where we ask for python if we can't find it
	if (zopeDialog->m_SetupWizard)
	{
		// Look for python.exe in instance dir
		if (zopeDialog->m_GlobalInterpreter == "" && !CFile::GetStatus(m_Path + "\\python.exe", status))
		{
			// Python was not found in instance dir
			m_InterpreterInInstance = FALSE;
			return IDD_ADD_INTERPRETER;
		}
		else
		{
			m_InterpreterInInstance = TRUE;
			
			// Look for z2.py in instance dir
			if (zopeDialog->m_GlobalStartupScript == "" && 
				!CFile::GetStatus(m_Path + "\\z2.py", status) &&
				!CFile::GetStatus(m_Path + "\\z2s.py", status))
			{
				m_StartupScriptInInstance = FALSE;
				return IDD_ADD_SOFTWARE;
			}
			else
			{
				m_StartupScriptInInstance = TRUE;
			}
		}
	
	}


	// Find out the next page to go to
	if (source == Copy)
		return IDD_ADD_FINISH;
	else if (source == CreateNew)
		return IDD_ADD_MANAGER;
	else if (source == AttachExisting)
		return IDD_ADD_FINISH;
		
	ASSERT(FALSE); // Shouldn't be here
	return CPropertyPage::OnWizardNext();
}


