// ControlPage.cpp : implementation file
//

#include "stdafx.h"
#include "ControlPanel.h"
#include "ControlPage.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

// Some user-defined states for the services
#define SERVICE_NOT_INSTALLED -1
#define SERVICE_INDETERMINATE -2


/////////////////////////////////////////////////////////////////////////////
// CControlPage property page

IMPLEMENT_DYNCREATE(CControlPage, CPropertyPage)

CControlPage::CControlPage() : CPropertyPage(CControlPage::IDD)
{
	fontBold = NULL;
	//{{AFX_DATA_INIT(CControlPage)
	//}}AFX_DATA_INIT
}

CControlPage::~CControlPage()
{
	delete fontBold;
}

void CControlPage::DoDataExchange(CDataExchange* pDX)
{
	CPropertyPage::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CControlPage)
	DDX_Control(pDX, IDC_MANAGE_ZOPE, m_ManageZope);
	DDX_Control(pDX, IDC_STOP, m_Stop);
	DDX_Control(pDX, IDC_STATE, m_State);
	DDX_Control(pDX, IDC_START, m_Start);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CControlPage, CPropertyPage)
	//{{AFX_MSG_MAP(CControlPage)
	ON_BN_CLICKED(IDC_START, OnStart)
	ON_BN_CLICKED(IDC_STOP, OnStop)
	ON_WM_TIMER()
	ON_BN_CLICKED(IDC_MANAGE_ZOPE, OnManageZope)
	//}}AFX_MSG_MAP

	/* This is the message our Windows95 zope service
	   reports back on.  The message id is initialised
	   in CControlPanelApp::OnInitialize() 
	 */
	ON_REGISTERED_MESSAGE(theApp.msgZopeResponse, OnZopeResponse)

END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CControlPage message handlers

void CControlPage::OnStart() 
{
	// Pointer to service
	ZopeService *service = zopeDialog->currentService;

	// Prompt to save changes first
	if (zopeDialog->m_Apply.IsWindowEnabled())
	{
		int nResult = MessageBox("You have made some changes to this configuration since you last saved.  Would you like to apply your changes before starting the server?", theApp.m_DialogCaption, MB_ICONQUESTION | MB_YESNOCANCEL);
		if (nResult == IDCANCEL)
			return;
		else if (nResult == IDYES)
			zopeDialog->ApplyChanges();
	}

	if (!(service->http[0] || service->https[0] || service->ftp[0] || service->webdav[0] || service->webdavs[0]))
	{
		int nResult = MessageBox("You have not enabled any ports for Zope to listen on.  Do you want to start Zope anyway?", theApp.m_DialogCaption, MB_ICONEXCLAMATION | MB_OKCANCEL);
		if (nResult == IDCANCEL)
			return;
	}
    
	if (zopeDialog->m_UseNTServiceDaemon) {
		// start up the NT-style service
		SC_HANDLE schService = OpenService(zopeDialog->schSCManager, service->m_ServiceName,
			SERVICE_ALL_ACCESS); 
 
		if (schService == NULL) {
			MessageBox(CString( (LPCSTR) IDS_ERROR_START_SERVICE_NOT_FOUND ), theApp.m_DialogCaption, MB_ICONEXCLAMATION);
		}
		if (!StartService(schService, 0, NULL)) { 
			MessageBox(CString( (LPCSTR) IDS_ERROR_START_CANT_START ), theApp.m_DialogCaption, MB_ICONEXCLAMATION);
		}
		CloseServiceHandle(schService); 

		
	} else {
		// Windows 95 style

		// Find the service95 application hwnd
		// Note that 32770 is the window class for dialog box
		HWND hwndServiceApp = ::FindWindow("#32770", CString("ZopeService95") + service->m_ServiceName);
		if (hwndServiceApp == NULL) {
			// The app is not yet running -- start it
			
			PROCESS_INFORMATION ps_info;

			STARTUPINFO s_info;
			ZeroMemory(&s_info, sizeof(s_info));
			s_info.cb = sizeof(s_info);

			int nResult = 0;

			CString cmdLine = "\"" + zopeDialog->m_Daemon95Path + "\" ";
			cmdLine += service->m_ServiceName;

			nResult = CreateProcess(NULL,					// app name
										cmdLine.GetBuffer(cmdLine.GetLength()),
															// command line
										NULL,				// process attributes
										NULL,				// thread attributes
										FALSE,				// inherit handles
										NORMAL_PRIORITY_CLASS,
															// creation flags
										NULL,				// environment block
										service->m_InstanceHome,		// working dir
										&s_info,			// startup info
										&ps_info);			// process info
			
			if (nResult == 0) {
				// could not start process
				//TODO msg
			} else {
				// give it a few seconds to start up
				SetStatus(SERVICE_START_PENDING);
				Sleep(3000);

				// Register cpl to receive status notifications
				if (!zopeDialog->m_UseNTServiceDaemon)
				{
					HWND hwndServiceApp = ::FindWindow("#32770", CString("ZopeService95") + service->m_ServiceName);
					::SendMessage(hwndServiceApp, theApp.msgZopeRegister, 0, (LPARAM) m_hWnd);
				}

			}
		} else {
			// App is already running -- tell it to start
			::SendMessage(hwndServiceApp, theApp.msgZopeStart, 0, 0);
			SetStatus(SERVICE_START_PENDING);
			Sleep(3000);
		}
	}

	UpdateStatus();
	
}

void CControlPage::OnStop() 
{
	// Pointer to service
	ZopeService *service = zopeDialog->currentService;

	if (zopeDialog->m_UseNTServiceDaemon) {
		// Use NT services

		SERVICE_STATUS ssStatus;

		SC_HANDLE schService = OpenService( 
			zopeDialog->schSCManager,          // SCM database 
			service->m_ServiceName,    // service name
			SERVICE_ALL_ACCESS); 
 
		if (schService == NULL) {
			MessageBox(CString( (LPCSTR) IDS_ERROR_STOP_SERVICE_NOT_FOUND ), theApp.m_DialogCaption, MB_ICONEXCLAMATION);
		}
		if (!ControlService(
				schService,  // handle to service 
				SERVICE_CONTROL_STOP,           
				&ssStatus)) {     // no arguments 
			MessageBox(CString( (LPCSTR) IDS_ERROR_STOP_CANT_CONTROL ), theApp.m_DialogCaption, MB_ICONEXCLAMATION);
		}

		CloseServiceHandle(schService);

	} else {
		// Do it in win95 style

		// Find the service95 application hwnd
		// Note that 32770 is the window class for dialog box
		HWND hwndServiceApp = ::FindWindow("#32770", CString("ZopeService95") + service->m_ServiceName);
		::SendMessage(hwndServiceApp, theApp.msgZopeStop, 0, 0);

		SetStatus(SERVICE_STOP_PENDING);

		// Give it some time to process
		Sleep(2000);
	}

	
	UpdateStatus();
	
}

BOOL CControlPage::OnSetActive() 
{
	if (zopeDialog->m_UseNTServiceDaemon)
	// Set up a timer to periodically poll the service manager
	// and ask if our service is running
	{	
		SetTimer(1, 5000, NULL); // Poll every 5 seconds
	}
	else
	// Register with the 95 service that we want to receive notifications of
	// start/stop
	{
		ZopeService *service = zopeDialog->currentService;
		
		if (service)
		{
			HWND hwndServiceApp = ::FindWindow("#32770", CString("ZopeService95") + service->m_ServiceName);
			::SendMessage(hwndServiceApp, theApp.msgZopeRegister, 0, (LPARAM) m_hWnd);
		}
	}
	
	UpdateStatus();

	return CPropertyPage::OnSetActive();
}

void CControlPage::UpdateStatus()
{
	// Update the status of this zope config
	ZopeService *currentService = zopeDialog->currentService;
	
	if (!currentService)
		return;

	if (zopeDialog->m_UseNTServiceDaemon)
	{
		// Use NT services
		SERVICE_STATUS ssStatus;
		SC_HANDLE schSCManager = zopeDialog->schSCManager;
		SC_HANDLE schService = OpenService(schSCManager,
										   currentService->m_ServiceName,
										   SERVICE_QUERY_STATUS);

		if (!schService)
		{
			SetStatus(SERVICE_NOT_INSTALLED);
			return;
		}

		if (!QueryServiceStatus(schService,
								&ssStatus))
		{
			SetStatus(SERVICE_INDETERMINATE);
			return;
		}

		SetStatus(ssStatus.dwCurrentState);
	}
	else
	{
		// Use Windows 95

		// Find the service app hwnd based on its title
		// Note that #32770 is the window class for a dlg box
		HWND hwndServiceApp = ::FindWindow("#32770", CString("ZopeService95") + currentService->m_ServiceName);
		if (hwndServiceApp == 0)
		{
			// The service app is not running
			SetStatus(SERVICE_STOPPED);
		}
		else
		{
			// Request that the servie app tell us its status
			// All Zope95 services communicate on the same MSG
			::SendMessage(hwndServiceApp, theApp.msgZopeQuery, 0, (LPARAM) m_hWnd);
		}

	}
}

void CControlPage::SetStatus(DWORD status)
{
	switch(status)
	{
	case SERVICE_STOPPED:
		m_State.SetWindowText("Stopped");
		EnableButtons(TRUE, FALSE);
		break;
	case SERVICE_START_PENDING:
		m_State.SetWindowText("Starting...");
		EnableButtons(FALSE, FALSE);
		break;
	case SERVICE_RUNNING:
		m_State.SetWindowText("Running");
		EnableButtons(FALSE, TRUE);
		break;
	case SERVICE_STOP_PENDING:
		m_State.SetWindowText("Stopping...");
		EnableButtons(FALSE, FALSE);
		break;
	case SERVICE_NOT_INSTALLED:
		m_State.SetWindowText("Service is not installed");
		EnableButtons(FALSE, FALSE);
		break;
	case SERVICE_INDETERMINATE:
		m_State.SetWindowText("Could not query service");
		EnableButtons(TRUE, TRUE);
		break;
	default:
		m_State.SetWindowText("Unknown state");
		EnableButtons(TRUE, TRUE);
		break;
	}
}

void CControlPage::EnableButtons(BOOL start, BOOL stop)
{
	m_Start.EnableWindow(start);
	m_Stop.EnableWindow(stop);
	m_ManageZope.EnableWindow(stop);
}


void CControlPage::OnZopeResponse(WPARAM wParam, LPARAM lParam)
{
	// This is the message callback through which the
	// 95 service sends its status

	// Status is contained in lParam
	SetStatus((DWORD) lParam);

}

BOOL CControlPage::OnKillActive() 
{
	

	if (zopeDialog->m_UseNTServiceDaemon)
	// Stop the timer
	{
		KillTimer(1);
	}
	else
	// Unregister cpl with service app to stop us receiving
	// status notifications.
	{
		ZopeService *service = zopeDialog->currentService;
		if (service)
		{
			HWND hwndServiceApp = ::FindWindow("#32770", CString("ZopeService95") + service->m_ServiceName);
			::SendMessage(hwndServiceApp, theApp.msgZopeRegister, 0, (LPARAM) m_hWnd);
		}
	}
	
	return CPropertyPage::OnKillActive();
}

void CControlPage::OnTimer(UINT nIDEvent) 
{
	// This timer is activated to poll for the current status
	UpdateStatus();
	
	CPropertyPage::OnTimer(nIDEvent);
}

BOOL CControlPage::OnInitDialog() 
{
	CPropertyPage::OnInitDialog();
	
	// Create the bold font
	fontBold = new CFont();
	fontBold->CreateFont(-11, 0, 0, 0, FW_BOLD,
						 FALSE, FALSE, FALSE,
						 ANSI_CHARSET,
						 OUT_DEFAULT_PRECIS, CLIP_DEFAULT_PRECIS,
						 DEFAULT_QUALITY, 
						 DEFAULT_PITCH | FF_DONTCARE,
						 "MS Shell Dlg");

	// Apply bold font to the status label
	m_State.SetFont(fontBold);
	
	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}

void CControlPage::OnManageZope() 
{
	ZopeService *service = zopeDialog->currentService;
	
	// Make sure there is a port available
	CString url;
	if (service->http[0])
		url = CString("http://127.0.0.1:") + service->http + CString("/manage");
	else if (service->https[0])
		url = CString("https://127.0.0.1:") + service->https + CString("/manage");
	else
	{
		MessageBox("You have not currently configured Zope to use the HTTP or Secure HTTP protocols, with which you need to manage Zope.  You can do this from the \"Services\" tab.", theApp.m_DialogCaption, MB_ICONEXCLAMATION);
		return;
	}

	ShellExecute(m_hWnd, "OPEN", url, "", "", SW_SHOW);


		
}
