// EmergencyUserPage.cpp : implementation file
//

#include "stdafx.h"
#include "ControlPanel.h"
#include "EmergencyUserPage.h"
#include "PasswordDialog.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CEmergencyUserPage property page

IMPLEMENT_DYNCREATE(CEmergencyUserPage, CPropertyPage)

CEmergencyUserPage::CEmergencyUserPage() : CPropertyPage(CEmergencyUserPage::IDD)
{
	//{{AFX_DATA_INIT(CEmergencyUserPage)
		// NOTE: the ClassWizard will add member initialization here
	//}}AFX_DATA_INIT
}

CEmergencyUserPage::~CEmergencyUserPage()
{
}

void CEmergencyUserPage::DoDataExchange(CDataExchange* pDX)
{
	CPropertyPage::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CEmergencyUserPage)
	DDX_Control(pDX, IDC_REMOVE, m_Remove);
	DDX_Control(pDX, IDC_CHANGE_PASSWORD, m_ChangePassword);
	DDX_Control(pDX, IDC_EMERGENCY_USER, m_EmergencyUser);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CEmergencyUserPage, CPropertyPage)
	//{{AFX_MSG_MAP(CEmergencyUserPage)
	ON_BN_CLICKED(IDC_REMOVE, OnRemove)
	ON_BN_CLICKED(IDC_CHANGE_PASSWORD, OnChangePassword)
	ON_WM_DESTROY()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CEmergencyUserPage message handlers

void CEmergencyUserPage::OnRemove() 
{
	// Delete the access file
	CFile::Remove(accessFilename);
	
	MessageBox("The emergency user was removed.", theApp.m_DialogCaption, MB_ICONINFORMATION);

	// Refresh display
	OnSetActive();
}

void CEmergencyUserPage::OnChangePassword() 
{
	// Show the password dialog
	CPasswordDialog dlg;
	dlg.defaultUsername = username;
	int nResult = dlg.DoModal();

	if (nResult == IDOK)
	{
		// Write details to the access file
		FILE * accessFile = fopen(accessFilename, "wt");
		fputs(dlg.m_Username + ":" + dlg.m_Password1, accessFile);
		fclose(accessFile);

		MessageBox("The emergency user has been updated.", theApp.m_DialogCaption, MB_ICONINFORMATION);

		// Update display
		OnSetActive();
	}
}

BOOL CEmergencyUserPage::OnSetActive() 
{
	// Open the access file to see if there is an
	// emergency user
	ZopeService *currentService = zopeDialog->currentService;

	// Provide a default username for creation
	username = "admin";

	if (currentService)
	{
		accessFilename = currentService->m_InstanceHome;
		if (accessFilename.Right(1) != "\\")
			accessFilename += "\\";
		accessFilename += "access";

		FILE *accessFile = fopen(accessFilename, "rt");
		if (!accessFile)
		{
			// Access file does not exist
			m_EmergencyUser.SetWindowText("Disabled");
			m_ChangePassword.SetWindowText("Create user...");
			m_Remove.EnableWindow(FALSE);
		}
		else
		{
			// Read the access file to determine username
			CString buf;
			fgets(buf.GetBuffer(256), 256, accessFile);
				// Arbitrary max length
			buf.ReleaseBuffer();
			
			int colonIndex = buf.Find(':');
			if (colonIndex != -1)
			{
				username = buf.Left(colonIndex);
				m_EmergencyUser.SetWindowText(username);
				m_ChangePassword.SetWindowText("Change password...");
				m_Remove.EnableWindow(TRUE);
			}
			else
			{
				// There was an error parsing the access
				// file
				m_EmergencyUser.SetWindowText("Error parsing file");
				m_ChangePassword.SetWindowText("Recreate user...");
				m_Remove.EnableWindow(TRUE);
			}
			fclose(accessFile);
		}
	}
			

	
	return CPropertyPage::OnSetActive();
}

void CEmergencyUserPage::OnDestroy() 
{
	CPropertyPage::OnDestroy();
	
	delete fontBold;
	
}

BOOL CEmergencyUserPage::OnInitDialog() 
{
	CPropertyPage::OnInitDialog();
	
	// Create the bold font
	fontBold = new CFont();
	fontBold->CreateFont(-11, 0, 0, 0, FW_BOLD,
						 FALSE, FALSE, FALSE,
						 ANSI_CHARSET,
						 OUT_DEFAULT_PRECIS, CLIP_DEFAULT_PRECIS,
						 DEFAULT_QUALITY, 
						 DEFAULT_PITCH | FF_DONTCARE,
						 "MS Shell Dlg");

	// Set to emergency user display
	m_EmergencyUser.SetFont(fontBold);
	
	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}
