// ExecutionPage.cpp : implementation file
//

#include "stdafx.h"
#include <objbase.h>	// Needed to create shortcut

#include "ControlPanel.h"
#include "ExecutionPage.h"
#include "AdvancedInstance.h"
#include "../common/NTServiceInfo.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

HRESULT CreateLink(LPCSTR lpszPathObj, LPCSTR lpszArg, LPSTR lpszWorkDir,
                         LPSTR lpszIcon, int IconIndex, LPSTR lpszPathLink,
                         LPCSTR lpszDesc);

/////////////////////////////////////////////////////////////////////////////
// CExecutionPage property page

IMPLEMENT_DYNCREATE(CExecutionPage, CPropertyPage)

CExecutionPage::CExecutionPage() : CPropertyPage(CExecutionPage::IDD)
{
	//{{AFX_DATA_INIT(CExecutionPage)
	m_Path = _T("");
	m_ServiceStartup = FALSE;
	m_ShowTrayIcon = FALSE;
	//}}AFX_DATA_INIT
}

CExecutionPage::~CExecutionPage()
{
}

void CExecutionPage::DoDataExchange(CDataExchange* pDX)
{
	CPropertyPage::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CExecutionPage)
	DDX_Control(pDX, IDC_SHOW_TRAY_ICON, m_btnShowTrayIcon);
	DDX_Control(pDX, IDC_PATH, m_editPath);
	DDX_Text(pDX, IDC_PATH, m_Path);
	DDX_Check(pDX, IDC_SERVICE_STARTUP, m_ServiceStartup);
	DDX_Check(pDX, IDC_SHOW_TRAY_ICON, m_ShowTrayIcon);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CExecutionPage, CPropertyPage)
	//{{AFX_MSG_MAP(CExecutionPage)
	ON_BN_CLICKED(IDC_BROWSE_PATH, OnBrowsePath)
	ON_BN_CLICKED(IDC_SERVICE_STARTUP, OnServiceStartupCheck)
	ON_BN_CLICKED(IDC_SHOW_TRAY_ICON, OnShowTrayIconCheck)
	ON_BN_CLICKED(IDC_ADVANCED, OnAdvanced)
	ON_BN_CLICKED(IDC_CREATE_START_MENU_SHORTCUT, OnCreateStartMenuShortcut)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CExecutionPage message handlers


void CExecutionPage::OnBrowsePath() 
{
	/* All of this code just to show
	   a silly browse-for-folder dialog! */
	
	char path[MAX_PATH];
	CString title((LPCSTR) IDS_PROMPT_BROWSE_INSTANCE);

	BROWSEINFO bi;
	ZeroMemory(&bi, sizeof(bi));
	bi.hwndOwner = m_hWnd;
	bi.lpszTitle = title;

	m_editPath.GetWindowText(path, MAX_PATH);

	ITEMIDLIST *idList;
	idList = SHBrowseForFolder(&bi);
	if (idList)
	{

        if ( SHGetPathFromIDList ( idList, path ) )
        {
			m_editPath.SetWindowText(path);
        }


		// Free mem
		IMalloc *imalloc = 0;
		if ( SUCCEEDED( SHGetMalloc ( &imalloc )) )
		{
			imalloc->Free(idList);
			imalloc->Release();
		}
	}
	
}



BOOL CExecutionPage::OnKillActive() 
{
	// Remember information but don't save to registry yet
	ZopeService *currentService = zopeDialog->currentService;
	
	if (currentService)
	{
		// Check if instance home matches client home

		CString NewInstanceHome;
		m_editPath.GetWindowText(NewInstanceHome);

		if (_stricmp(CZopeDialog::AddTrailingSlash(currentService->m_InstanceHome) + "var\\", CZopeDialog::AddTrailingSlash(m_ClientHome)) == 0
			&& _stricmp(CZopeDialog::AddTrailingSlash(currentService->m_InstanceHome), CZopeDialog::AddTrailingSlash(NewInstanceHome)) != 0)
		{
			// They used to.. prompt to change client home as well
			if (MessageBox("Your change to Instance Home will probably affect Client Home as well.  Do you want to update Client Home to reflect the new Instance Home?\n\nIf you are unsure, choose Yes.", theApp.m_DialogCaption, MB_ICONQUESTION | MB_YESNO) == IDYES)
			{
				m_ClientHome = CZopeDialog::AddTrailingSlash(NewInstanceHome) + "var";
			}
		}
		
		UpdateData(TRUE);
		currentService->m_InstanceHome = m_Path;
		currentService->m_Interpreter = m_Interpreter;
		currentService->m_StartupScript = m_StartupScript;
		currentService->m_ClientHome = m_ClientHome;
		currentService->m_PythonHome = m_PythonHome;
		currentService->m_SoftwareHome = m_SoftwareHome;
		currentService->m_SystemStartup = m_ServiceStartup;
		currentService->m_ShowTrayIcon = m_ShowTrayIcon;
		currentService->m_UseGlobalDefaults = !m_Override;
	}

	return CPropertyPage::OnKillActive();
}

BOOL CExecutionPage::OnSetActive() 
{

	ZopeService *currentService = zopeDialog->currentService;

	if (currentService)
	{
		// Retrieve values from ZopeService class
		m_Path = currentService->m_InstanceHome;
		m_Interpreter = currentService->m_Interpreter;
		m_StartupScript = currentService->m_StartupScript;
		m_PythonHome = currentService->m_PythonHome;
		m_ClientHome = currentService->m_ClientHome;
		m_SoftwareHome = currentService->m_SoftwareHome;
		m_ServiceStartup = currentService->m_SystemStartup;
		m_ShowTrayIcon = currentService->m_ShowTrayIcon;
		m_Override = !currentService->m_UseGlobalDefaults;
		

		if (zopeDialog->m_UseNTServiceDaemon)
		{
			// Can't show a tray icon for a service
			// Well, not yet anyway.
			m_ShowTrayIcon = FALSE;
			m_btnShowTrayIcon.EnableWindow(FALSE);
		}
		else
		{
			m_btnShowTrayIcon.EnableWindow(TRUE);
		}

		UpdateData(FALSE);	
	}

	
	return CPropertyPage::OnSetActive();
}

void CExecutionPage::OnServiceStartupCheck() 
{
	// Enable the apply button
	ASSERT(zopeDialog);
	zopeDialog->m_Apply.EnableWindow(TRUE);
}

void CExecutionPage::OnShowTrayIconCheck() 
{
	// Enable the apply button
	ASSERT(zopeDialog);
	zopeDialog->m_Apply.EnableWindow(TRUE);
}

void CExecutionPage::OnAdvanced() 
{
	CAdvancedInstance dlg;
	dlg.m_Clienthome = m_ClientHome;
	dlg.m_Startupscript = m_StartupScript;
	dlg.m_Softwarehome = m_SoftwareHome;
	dlg.m_Interpreter = m_Interpreter;
	dlg.m_Pythonhome = m_PythonHome;
	dlg.m_Override = m_Override;

	int nResult = dlg.DoModal();

	if (nResult == IDOK)
	{
		m_ClientHome = dlg.m_Clienthome;
		m_StartupScript = dlg.m_Startupscript;
		m_SoftwareHome = dlg.m_Softwarehome;
		m_Interpreter = dlg.m_Interpreter;
		m_PythonHome = dlg.m_Pythonhome;
		m_Override = dlg.m_Override;

		// Enable the apply button
		ASSERT(zopeDialog);
		zopeDialog->m_Apply.EnableWindow(TRUE);
	}

}

void CExecutionPage::OnCreateStartMenuShortcut() 
{
	ZopeService *service = zopeDialog->currentService;

	char lnkPath[MAX_PATH];
	SHGetSpecialFolderPath(m_hWnd, lnkPath, CSIDL_PROGRAMS, 0);
		
	CString lnkTarget;
	if (zopeDialog->m_UseNTServiceDaemon)
		lnkTarget = "\"" + zopeDialog->m_DaemonNTPath + "\"";
	else
		lnkTarget = "\"" + zopeDialog->m_Daemon95Path + "\"";

	if (!SUCCEEDED(CreateLink(lnkTarget, service->m_ServiceName, "", "", 0, lnkPath, service->m_ServiceName)))
		MessageBox("Could not create shortcut", theApp.m_DialogCaption, MB_ICONEXCLAMATION);

}


// Taken straight from the doco.
HRESULT CreateLink(LPCSTR lpszPathObj, LPCSTR lpszArg, LPSTR lpszWorkDir,
                         LPSTR lpszIcon, int IconIndex, LPSTR lpszPathLink,
                         LPCSTR lpszDesc)
{
  HRESULT hres;
  IShellLink* psl;

  CoInitialize(NULL);

  // Get a pointer to the IShellLink interface.
  hres = CoCreateInstance((_GUID)CLSID_ShellLink, NULL,
                          CLSCTX_INPROC_SERVER,
                          (_GUID)IID_IShellLink,
                          (void**)&psl);
  if (SUCCEEDED(hres))
  {
    IPersistFile* ppf;

    // Set the path to the shortcut target, and add the
    // description.
    psl->SetPath(lpszPathObj);
    psl->SetArguments(lpszArg);
    psl->SetWorkingDirectory(lpszWorkDir);
    psl->SetIconLocation(lpszIcon,IconIndex);
    psl->SetDescription(lpszDesc);

    // Query IShellLink for the IPersistFile interface
    //for saving the shortcut in persistent storage.
    hres = psl->QueryInterface((_GUID)IID_IPersistFile,(void**)&ppf);

    if (SUCCEEDED(hres))
    {
      WORD wsz[MAX_PATH];

      // Ensure that the string is ANSI.
      MultiByteToWideChar(CP_ACP, 0, lpszPathLink, -1,
                          (wchar_t*)wsz, MAX_PATH);

      // Save the link by calling IPersistFile::Save.
      hres = ppf->Save((wchar_t*)wsz, TRUE);
      ppf->Release();
    }
    psl->Release();
  }

  return hres;
}


