// WizardPage.cpp : implementation file
//

#include "stdafx.h"
#include "ControlPanel.h"
#include "WizardPage.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CWizardPage property page

IMPLEMENT_DYNCREATE(CWizardPage, CPropertyPage)

CWizardPage::CWizardPage(int dlgId) : CPropertyPage(dlgId)
{
	//{{AFX_DATA_INIT(CWizardPage)
		// NOTE: the ClassWizard will add member initialization here
	//}}AFX_DATA_INIT
}

CWizardPage::CWizardPage()
{
	// Should never be called this way
	CWizardPage(0);
}

CWizardPage::~CWizardPage()
{
}

void CWizardPage::DoDataExchange(CDataExchange* pDX)
{
	CPropertyPage::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CWizardPage)
		// NOTE: the ClassWizard will add DDX and DDV calls here
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CWizardPage, CPropertyPage)
	//{{AFX_MSG_MAP(CWizardPage)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CWizardPage message handlers

void CWizardPage::SetButtons(int flags)
{
	CPropertySheet* parent = (CPropertySheet*)GetParent();
	parent->SetWizardButtons(flags);
}

BOOL CWizardPage::OnInitDialog() 
{
	CPropertyPage::OnInitDialog();
	
	// Load the logo bitmap from a resource
	m_bmpLogo.LoadBitmap(IDB_LOGO);

	// Create the transparency mask needed for the Zope logo
	PrepareMask( &m_bmpLogo, &m_bmpMask, NULL);	
	
	
	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}

void CWizardPage::Paint(CPaintDC &dc) 
{
	// Set dialog caption -- this is a dodgy place to put
	// this but I can't seem to get it to work any where else
	GetParent()->SetWindowText( CString((LPCSTR) IDS_CAPTION_WIZARD) );
	
	// Get the rectangle for where to draw the logo
	CRect rc;
	CWnd *placeholder = GetDlgItem(IDC_LOGO_PLACEHOLDER);
	placeholder->GetWindowRect(rc);
	ScreenToClient(rc);
	
	// Get the width & height of bitmap
	BITMAP bmp;
	m_bmpLogo.GetBitmap(&bmp);


	CDC dcMem;                  // memory device context
	dcMem.CreateCompatibleDC(&dc);

	// Select the bmp into the tmp memory DC
	CBitmap* pOldBmp = (CBitmap*) dcMem.SelectObject(&m_bmpLogo);

	DrawTransparentBitmap( &dc,           // The destination DC.
						 rc.left,         // Where to draw
						 rc.top,
						 bmp.bmWidth,     // Width & Height
						 bmp.bmHeight,
						 &dcMem,        // the DC holding the bmp
						 0, // x & y pos in master bmp
						 0);

	dcMem.SelectObject(pOldBmp);
}

void CWizardPage::PrepareMask( CBitmap* pBmpSource,
							  CBitmap* pBmpMask,
							  COLORREF clrpTransColor, // Pass null if unknown
							  int iTransPixelX,      // = 0
							  int iTransPixelY       // = 0
							)
{
   BITMAP bm;

   // Get the dimensions of the source bitmap
   pBmpSource->GetObject(sizeof(BITMAP), &bm);

   // Create the mask bitmap
   pBmpMask->DeleteObject();
   pBmpMask->CreateBitmap( bm.bmWidth, bm.bmHeight, 1, 1, NULL);

   // We will need two DCs to work with. One to hold the Image
   // (the source), and one to hold the mask (destination).
   // When blitting onto a monochrome bitmap from a color, pixels
   // in the source color bitmap that are equal to the background
   // color are blitted as white. All the remaining pixels are
   // blitted as black.

   CDC hdcSrc, hdcDst;

   hdcSrc.CreateCompatibleDC(NULL);
   hdcDst.CreateCompatibleDC(NULL);

   // Load the bitmaps into memory DC
   CBitmap* hbmSrcT = (CBitmap*) hdcSrc.SelectObject(pBmpSource);
   CBitmap* hbmDstT = (CBitmap*) hdcDst.SelectObject(pBmpMask);

   // Dynamically get the transparent color
   COLORREF clrTrans;
   if (clrpTransColor == NULL)
   {
      // User did not specify trans color so get it from bmp
      clrTrans = hdcSrc.GetPixel(iTransPixelX, iTransPixelY);
   }
   else
   {
      clrTrans = clrpTransColor;
   }


   // Change the background to trans color
   COLORREF clrSaveBk  = hdcSrc.SetBkColor(clrTrans);

   // This call sets up the mask bitmap.
   hdcDst.BitBlt(0,0,bm.bmWidth, bm.bmHeight, &hdcSrc,0,0,SRCCOPY);

   // Now, we need to paint onto the original image, making
   // sure that the "transparent" area is set to black. What
   // we do is AND the monochrome image onto the color Image
   // first. When blitting from mono to color, the monochrome
   // pixel is first transformed as follows:
   // if  1 (black) it is mapped to the color set by SetTextColor().
   // if  0 (white) is is mapped to the color set by SetBkColor().
   // Only then is the raster operation performed.

   COLORREF clrSaveDstText = hdcSrc.SetTextColor(RGB(255,255,255));
   hdcSrc.SetBkColor(RGB(0,0,0));

   hdcSrc.BitBlt(0,0,bm.bmWidth, bm.bmHeight, &hdcDst,0,0,SRCAND);

   // Clean up by deselecting any objects, and delete the
   // DC's.
   hdcDst.SetTextColor(clrSaveDstText);

   hdcSrc.SetBkColor(clrSaveBk);
   hdcSrc.SelectObject(hbmSrcT);
   hdcDst.SelectObject(hbmDstT);

   hdcSrc.DeleteDC();
   hdcDst.DeleteDC();
}



void CWizardPage::DrawTransparentBitmap(CDC* pDC,
									   int xStart,  int yStart,
									   int wWidth,  int wHeight,
									   CDC* pTmpDC,
									   int xSource, // = 0
									   int ySource  // = 0
									  )
{

   // We are going to paint the two DDB's in sequence to the destination.
   // 1st the monochrome bitmap will be blitted using an AND operation to
   // cut a hole in the destination. The color image will then be ORed
   // with the destination, filling it into the hole, but leaving the
   // surrounding area untouched.

   CDC hdcMem;
   hdcMem.CreateCompatibleDC(NULL);

   CBitmap* hbmT = hdcMem.SelectObject(&m_bmpMask);

   pDC->BitBlt( xStart, yStart, wWidth, wHeight, &hdcMem,
                xSource, ySource, SRCAND);

   // Also note the use of SRCPAINT rather than SRCCOPY.

   pDC->BitBlt(xStart, yStart, wWidth, wHeight, pTmpDC,
               xSource, ySource,SRCPAINT);

   // Now, clean up.
   hdcMem.SelectObject(hbmT);
   hdcMem.DeleteDC();
}

