// ZopeDialog.cpp : implementation file
//

#include "stdafx.h"

#include <direct.h> // for _getcwd()

#include "ControlPanel.h"
#include "ZopeDialog.h"
#include "NewServiceDialog.h"
#include "AdvancedDialog.h"

// Wizard pages
#include "AddName.h"
#include "AddSource.h"
#include "AddInstanceDir.h"
#include "AddInterpreter.h"
#include "AddSoftware.h"
#include "AddManager.h"
#include "AddFinish.h"

IMPLEMENT_DYNAMIC(CZopeDialog, CDialog);

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CZopeDialog dialog


CZopeDialog::CZopeDialog(CWnd* pParent /*=NULL*/)
	: CDialog(CZopeDialog::IDD, pParent)
{
	currentService = NULL;
	servicesPage = NULL;
	logPage = NULL;
	aboutPage = NULL;
	executionPage = NULL;
	emergencyUserPage = NULL;
	controlPage = NULL;
	propertySheet = NULL;
	schSCManager = NULL;
	m_SetupWizard = FALSE;
	m_SingleInstanceMode = FALSE;
	//{{AFX_DATA_INIT(CZopeDialog)
	//}}AFX_DATA_INIT
}


void CZopeDialog::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CZopeDialog)
	DDX_Control(pDX, IDOK, m_Apply);
	DDX_Control(pDX, IDC_CONFIGURATION_DELETE, m_ConfigurationDelete);
	DDX_Control(pDX, IDC_CONFIGURATION_LIST, m_ConfigurationList);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CZopeDialog, CDialog)
	//{{AFX_MSG_MAP(CZopeDialog)
	ON_BN_CLICKED(IDC_CONFIGURATION_ADD, OnConfigurationAdd)
	ON_BN_CLICKED(IDC_CONFIGURATION_DELETE, OnConfigurationDelete)
	ON_CBN_SELCHANGE(IDC_CONFIGURATION_LIST, OnSelchangeConfigurationList)
	ON_WM_DESTROY()
	ON_BN_CLICKED(IDC_ADVANCED, OnAdvanced)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CZopeDialog message handlers

void CZopeDialog::OnConfigurationAdd() 
{

	// Launch the wizard
	CPropertySheet wizard(IDS_CAPTION_WIZARD, this);
	CAddName pageName;
	CAddSource pageSource;
	CAddInstanceDir pageInstance;
	CAddManager pageManager;
	CAddFinish pageFinish;

	pageName.zopeDialog = this;
	pageSource.zopeDialog = this;
	pageInstance.zopeDialog = this;
	pageManager.zopeDialog = this;
	pageFinish.zopeDialog = this;

	wizard.AddPage(&pageName);
	wizard.AddPage(&pageSource);
	wizard.AddPage(&pageInstance);
	wizard.AddPage(&pageManager);
	wizard.AddPage(&pageFinish);
	wizard.SetWizardMode();
	int nResult = wizard.DoModal();
  
	if (nResult == ID_WIZFINISH)
	{

		// Create a new service
		ZopeService *newService = new ZopeService(theApp.servicesKey, 
												  pageName.m_Name, 
												  schSCManager, 
												  m_GlobalSoftwareHome,
												  m_GlobalStartupScript,
												  m_GlobalPythonHome,
												  m_GlobalInterpreter);

		// Get base service
		ZopeService *baseService = pageSource.m_BaseInstance;

		if (pageSource.m_SourceType == DontCopy)
		{
			newService->m_InstanceHome = baseService->m_InstanceHome;
			newService->m_ClientHome = baseService->m_ClientHome;
		}
		else
		{
			newService->m_InstanceHome = pageInstance.m_Path;
			newService->m_ClientHome = AddTrailingSlash(pageInstance.m_Path) + "var";
		}

		if (pageSource.m_SourceType != CreateNew && pageSource.m_SourceType != AttachExisting)
		{
			// Copy all the data from the base service to the new service
			strcpy(newService->ftp, baseService->ftp);
			strcpy(newService->http, baseService->http);
			strcpy(newService->https, baseService->https);
			strcpy(newService->webdav, baseService->webdav);
			strcpy(newService->webdavs, baseService->webdavs);
			
			newService->m_Interpreter = baseService->m_Interpreter;
			newService->m_StartupScript = baseService->m_StartupScript;
			newService->m_ClientHome = baseService->m_ClientHome;
			newService->m_PythonHome = baseService->m_PythonHome;
			newService->m_SoftwareHome = baseService->m_SoftwareHome;
			newService->m_UseGlobalDefaults = baseService->m_UseGlobalDefaults;
			newService->m_ShowTrayIcon = baseService->m_ShowTrayIcon;
			newService->m_SystemStartup = baseService->m_SystemStartup;
			newService->m_StdOutLogFile = baseService->m_StdOutLogFile;
			newService->m_StupidLogFile = baseService->m_StupidLogFile;
						
		}

		if (pageSource.m_SourceType == Copy)
		{	
			// Copy the files across

			/*  There is an awful lot of kerfuffle going on here
			    because I had absolutely no luck getting SHFileOperation
				to work with FOF_MULTIDESTFILES... it just seems to
				ignore the flag ?!? */

			SHFILEOPSTRUCT copyStruct;
			ZeroMemory(&copyStruct, sizeof(copyStruct));

			copyStruct.hwnd = m_hWnd;
			copyStruct.wFunc = FO_COPY;
			copyStruct.fFlags = FOF_NOCONFIRMMKDIR | FOF_SIMPLEPROGRESS;
			copyStruct.hNameMappings = NULL;
			
			CString srcDir = AddTrailingSlash(baseService->m_InstanceHome);
			CString destDir = AddTrailingSlash(newService->m_InstanceHome);

			char *pFrom = new char[3 * srcDir.GetLength() + 100];
			char *pTo = new char[3 * destDir.GetLength() + 100];
			copyStruct.pFrom = pFrom;
			copyStruct.pTo = pTo;		// Necessary to get around const
			
			// Copy Products dir first
			CFileStatus status;
			if (CFile::GetStatus(srcDir + "products", status))
			{
				strcpy(pFrom, srcDir + "products\\*.*");
				pFrom[strlen(pFrom) + 1] = 0;  // Finish with double null
				strcpy(pTo, destDir + "products\\");
				pTo[strlen(pTo) + 1] = 0;  // Finish with double null
			}

			copyStruct.lpszProgressTitle = "Copying Zope products...";
			SHFileOperation(&copyStruct);
			
			// Copy import dir
			if (CFile::GetStatus(srcDir + "import", status))
			{
				strcpy(pFrom, srcDir + "import\\*.*");
				pFrom[strlen(pFrom) + 1] = 0;  // Finish with double null
				strcpy(pTo, destDir + "import\\");
				pTo[strlen(pTo) + 1] = 0;  // Finish with double null
			}

			copyStruct.lpszProgressTitle = "Copying Zope imports...";
			SHFileOperation(&copyStruct);

			// Copy var dir
			strcpy(pFrom, AddTrailingSlash(baseService->m_ClientHome) + "*.*");
			pFrom[strlen(pFrom) + 1] = 0;  // Finish with double null
			strcpy(pTo, AddTrailingSlash(newService->m_ClientHome));
			pTo[strlen(pTo) + 1] = 0;  // Finish with double null

			copyStruct.lpszProgressTitle = "Copying Zope database...";
			SHFileOperation(&copyStruct);
			
			// Copy pem files
			strcpy(pFrom, srcDir + "*.pem");
			pFrom[strlen(pFrom) + 1] = 0;  // Finish with double null
			strcpy(pTo, destDir);
			pTo[strlen(pTo) + 1] = 0;  // Finish with double null

			copyStruct.lpszProgressTitle = "Copying certificates...";
			SHFileOperation(&copyStruct);

			delete pFrom;
			delete pTo;
		}

		else if (pageSource.m_SourceType == CreateNew)
		{
			// Create the instance dir and a Var dir beneath it
			CreateDirectory(newService->m_InstanceHome, NULL);
			CreateDirectory(newService->m_ClientHome, NULL);
		}

		if (pageSource.m_SourceType == CreateNew)
		{
			// Create initial manager
			FILE *fInitUser = fopen(AddTrailingSlash(newService->m_InstanceHome) +  "inituser", "wt");
			if (!fInitUser)
				MessageBox("Warning: Unable to create initial manager.", theApp.m_DialogCaption, MB_ICONEXCLAMATION);
			else
			{
				fprintf(fInitUser, "%s:%s", pageManager.m_Username, pageManager.m_Password1);
				fclose(fInitUser);
			}
		}

		// Save it to registry
		BOOL bResult;
		if (m_UseNTServiceDaemon)
			bResult = newService->Save("\"" + m_DaemonNTPath + "\"");
		else
			bResult = newService->Save("\"" + m_Daemon95Path + "\"");

		
		if (!bResult)
		{
			// Something went wrong TODO error checking
			MessageBox("The service could not be created.  It is possible that a service with this name already exists, or has been marked for deletion.  You may want to try restarting your computer to fix the problem, or renaming the service.", theApp.m_DialogCaption, MB_ICONEXCLAMATION);
			newService->Remove();
			delete newService;
			return;
		}

		// Add it to the drop-down list
		int index = m_ConfigurationList.AddString(pageName.m_Name);
		m_ConfigurationList.SetItemDataPtr(index, newService);

		m_ConfigurationList.EnableWindow(TRUE);
		m_ConfigurationDelete.EnableWindow(TRUE);

		// Ensure all the tabs are showing
		ShowAllTabs();

		// Set it as the current configuration
		m_ConfigurationList.SetCurSel(index);
	}
	
	OnSelchangeConfigurationList();

  
}

void CZopeDialog::OnConfigurationDelete() 
{
	int nResult = MessageBox("Are you sure you want to permanently remove this service configuration?", theApp.m_DialogCaption, MB_ICONEXCLAMATION | MB_OKCANCEL);
	if (nResult == IDOK)
	{
		// Get current selection
		int sel = m_ConfigurationList.GetCurSel();
		ZopeService *service = (ZopeService *) m_ConfigurationList.GetItemDataPtr(sel);

		// Remove service from list
		m_ConfigurationList.DeleteString(sel);
		
		// Remove service from reg
		service->Remove();

		// Clean up memory
		delete service;
		currentService = NULL;

		// Select another service if one exists
		if (m_ConfigurationList.GetCount() > 0)
			m_ConfigurationList.SetCurSel(0);
		else
		{
			// Disable controls
			m_ConfigurationList.EnableWindow(FALSE);
			m_ConfigurationDelete.EnableWindow(FALSE);
		}

		MessageBox("You will have to manually remove this configuration's instance directory, if desired.", theApp.m_DialogCaption, MB_ICONINFORMATION);
	}

	OnSelchangeConfigurationList();		
	
}

void CZopeDialog::OnSelchangeConfigurationList() 
{
	CPropertyPage *activePage = propertySheet->GetActivePage();
	ASSERT(activePage);
	
	// Save current data to memory (not registry)
	activePage->OnKillActive();

	// Change the current selection
	int sel = m_ConfigurationList.GetCurSel();
	if (sel != CB_ERR)
	{
		currentService = (ZopeService *) m_ConfigurationList.GetItemDataPtr(sel);
		
		// Cause selected tab (page) to reload its data
		// from the new ZopeService class
		activePage->OnSetActive();
	}
	else
	{
		// Kill all the tabs and show a help screen
		ShowHelpTab();
	}
	
}

BOOL CZopeDialog::OnInitDialog() 
{
	CDialog::OnInitDialog();
	
	// Get from the registry in HKEY_LOCAL_MACHINE\regKey\Services\...

	HKEY hZopeKey;
	DWORD disposition;	// Ignored
	int success = ERROR_SUCCESS;
		
	RegCreateKeyEx(HKEY_LOCAL_MACHINE, theApp.regKey, NULL, "", NULL, KEY_READ, NULL, &hZopeKey, &disposition);

	// Restore window position
	RECT pos;
	GetWindowRect(&pos);		// In case reg values have not been set
	DWORD size = sizeof(pos.left);
	DWORD type;
	RegQueryValueEx(hZopeKey, "WindowX", NULL, &type, (BYTE *) &pos.left, &size);
	RegQueryValueEx(hZopeKey, "WindowY", NULL, &type, (BYTE *)  &pos.top, &size);
	SetWindowPos(NULL, pos.left, pos.top, NULL, NULL, SWP_NOZORDER | SWP_NOSIZE);
	
	// Are we running in single instance mode?
	if (RegQueryValueEx(hZopeKey, "SingleInstanceMode", NULL, &type, NULL, &size) == ERROR_SUCCESS)
	{
		// Yes we are -- hide the combo box etc
		m_SingleInstanceMode = TRUE;
		
		m_ConfigurationList.ShowWindow(SW_HIDE);
		m_ConfigurationDelete.ShowWindow(SW_HIDE);
		GetDlgItem(IDC_STATIC_ZOPE_CONFIGURATION)->ShowWindow(SW_HIDE);
		GetDlgItem(IDC_CONFIGURATION_ADD)->ShowWindow(SW_HIDE);

		// Move the controls up to fill in the gap
		const int PIXELS_UP = 32;
		CRect rectWindow;
		GetDlgItem(IDC_PROPERTYSHEET)->GetWindowRect(rectWindow);
		ScreenToClient(rectWindow);
		GetDlgItem(IDC_PROPERTYSHEET)->SetWindowPos(NULL, rectWindow.left, rectWindow.top - PIXELS_UP, 0, 0, SWP_NOSIZE | SWP_NOZORDER);
		GetDlgItem(IDC_ADVANCED)->GetWindowRect(rectWindow);
		ScreenToClient(rectWindow);
		GetDlgItem(IDC_ADVANCED)->SetWindowPos(NULL, rectWindow.left, rectWindow.top - PIXELS_UP, 0, 0, SWP_NOSIZE | SWP_NOZORDER);
		GetDlgItem(IDOK)->GetWindowRect(rectWindow);
		ScreenToClient(rectWindow);
		GetDlgItem(IDOK)->SetWindowPos(NULL, rectWindow.left, rectWindow.top - PIXELS_UP, 0, 0, SWP_NOSIZE | SWP_NOZORDER);
		GetDlgItem(IDCANCEL)->GetWindowRect(rectWindow);
		ScreenToClient(rectWindow);
		GetDlgItem(IDCANCEL)->SetWindowPos(NULL, rectWindow.left, rectWindow.top - PIXELS_UP, 0, 0, SWP_NOSIZE | SWP_NOZORDER);

		// Resize main window
		GetWindowRect(rectWindow);
		SetWindowPos(NULL, 0, 0, rectWindow.Width(), rectWindow.Height() - PIXELS_UP, SWP_NOMOVE | SWP_NOZORDER);

	}
	


	// Read global variables
	char CurrentDir[MAX_PATH];
	GetCurrentDirectory(MAX_PATH, CurrentDir);
	
	m_GlobalSoftwareHome = GetSetting(hZopeKey, "SoftwareHome", CurrentDir);
	AddTrailingSlash(&m_GlobalSoftwareHome);
	m_GlobalStartupScript = GetSetting(hZopeKey, "StartupScript", m_GlobalSoftwareHome + "z2.py");
	m_GlobalPythonHome = GetSetting(hZopeKey, "PythonHome", CurrentDir);
	AddTrailingSlash(&m_GlobalPythonHome);
	m_GlobalInterpreter = GetSetting(hZopeKey, "Interpreter", m_GlobalPythonHome + "python.exe");
	m_DefaultInstanceDir = GetSetting(hZopeKey, "DefaultInstanceDir", CurrentDir);
	
	// Check the system type -- is a service manager available? 
	schSCManager = OpenSCManager(NULL, NULL, SC_MANAGER_ALL_ACCESS); 
 	m_ServiceManagerAvailable = (schSCManager != NULL);
	
	// Even if we are running NT the user may have elected
	// to use 95 services
	DWORD value;
	size = sizeof(value);
	success = RegQueryValueEx(hZopeKey, "UseNTServiceDaemon", NULL, &type, (BYTE *) &value, &size);
	if (success == ERROR_SUCCESS)
		m_UseNTServiceDaemon = value;
	else
		m_UseNTServiceDaemon = m_ServiceManagerAvailable;	// Default seems reasonable

	if (!m_UseNTServiceDaemon)
	{
		// Close the service handle if we opened it
		// unnecessarily
		if (schSCManager)
			CloseServiceHandle(schSCManager);
		schSCManager = NULL;	// Don't want to go and use
								// it accidently!
	}
	
	// Initialize the list of services
	HKEY hServicesKey;
	RegCreateKeyEx(hZopeKey, "Services", NULL, "", NULL, KEY_READ, NULL, &hServicesKey, &disposition);
	
	// Make sure there is a services for single-instance mode
	if (m_SingleInstanceMode)
	{
		DWORD numSubKeys;
		RegQueryInfoKey(hServicesKey, NULL, NULL, NULL, &numSubKeys, NULL, NULL, NULL, NULL, NULL, NULL, NULL);
		if (numSubKeys == 0)
			// Add an instance
			DoSetupWizard();
	}
	
	// Enumerate through all services
	int index = 0;
	while (success != ERROR_NO_MORE_ITEMS)
	{
		char name[1024];
		DWORD size = sizeof(name);
		FILETIME filetime;			// Ignored
		success = RegEnumKeyEx(hServicesKey, index++, name, &size, NULL, NULL, NULL, &filetime);
		if (success == ERROR_SUCCESS)
		{
			// Create a ZopeService object for this service to handle
			// its configuration and add it to the list
			ZopeService *newService = new ZopeService(theApp.servicesKey,
													  name,
													  schSCManager,
													  m_GlobalSoftwareHome,
													  m_GlobalStartupScript,
													  m_GlobalPythonHome,
													  m_GlobalInterpreter
													 );
			
			int listIndex = m_ConfigurationList.AddString(name);
			m_ConfigurationList.SetItemDataPtr(listIndex, newService);

		}
	}
		
	// Disable the list and delete buttons if list is empty
	if (m_ConfigurationList.GetCount() == 0)
	{
		m_ConfigurationList.EnableWindow(FALSE);
		m_ConfigurationDelete.EnableWindow(FALSE);
	}
	else
	{
		// Select a default service
		char name[1024];
		DWORD size = sizeof(name);
		DWORD type;
		success = RegQueryValueEx(hZopeKey, "DefaultService", NULL, &type, (BYTE *) name, &size);
		if (success == ERROR_SUCCESS)
		{
			if (m_ConfigurationList.SelectString(-1, name) == CB_ERR)
				m_ConfigurationList.SetCurSel(0);
		} else
			m_ConfigurationList.SetCurSel(0);
	}


	// Create the tab control (CPropertySheet)
	propertySheet = new CPropertySheet(theApp.m_DialogCaption);

	if (m_ConfigurationList.GetCount() > 0)
	{
		ShowAllTabs();
	}
	else
	{
		ShowHelpTab();
	}

	propertySheet->Create(this, WS_CHILD | WS_VISIBLE, 0);
	propertySheet->ModifyStyleEx(0, WS_EX_CONTROLPARENT);
	propertySheet->ModifyStyle(0, WS_TABSTOP);

	// Align property sheet to the correct coordinates on the dialog
	CRect rcSheet;
	GetDlgItem( IDC_PROPERTYSHEET )->GetWindowRect(&rcSheet);
	ScreenToClient(&rcSheet);
	propertySheet->SetWindowPos(NULL, rcSheet.left - 7, rcSheet.top - 7, 0, 0,
						SWP_NOZORDER | SWP_NOSIZE | SWP_NOACTIVATE);

	// Now we are ready to show data on the first page
	OnSelchangeConfigurationList();

	// Read path of NT Daemon
	size = MAX_PATH;
	success = RegQueryValueEx(hZopeKey, "DaemonNTPath", NULL, &type, (BYTE *) m_DaemonNTPath.GetBuffer(size), &size);
	m_DaemonNTPath.ReleaseBuffer();
	if (success != ERROR_SUCCESS)
	{
		// Provide a default value
		_getcwd(m_DaemonNTPath.GetBuffer(MAX_PATH), MAX_PATH);
		m_DaemonNTPath.ReleaseBuffer();
		AddTrailingSlash(&m_DaemonNTPath);
		m_DaemonNTPath += "ZopeServiceNT.exe";
	}

	// Read path of 95 Daemon
	size = MAX_PATH;
	success = RegQueryValueEx(hZopeKey, "Daemon95Path", NULL, &type, (BYTE *) m_Daemon95Path.GetBuffer(size), &size);
	m_Daemon95Path.ReleaseBuffer();
	if (success != ERROR_SUCCESS)
	{
		// Provide a default value
		_getcwd(m_Daemon95Path.GetBuffer(MAX_PATH), MAX_PATH);
		m_Daemon95Path.ReleaseBuffer();
		AddTrailingSlash(&m_Daemon95Path);
		m_Daemon95Path += "ZopeService95.exe";
	}

	

	RegCloseKey(hServicesKey);
	RegCloseKey(hZopeKey);

	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}


CString CZopeDialog::GetSetting(HKEY key, LPCSTR setting, CString defaultSetting)
{
	// This function reads a value tied to <setting> from the registry in <key>.
	// If it doesn't exist return <defaultSetting>

	LONG success;
	DWORD size;
	CString value;

	// Get the size of the value
	success = RegQueryValueEx(key, setting, NULL, NULL, NULL, &size);

	if (success != ERROR_SUCCESS)
		return defaultSetting;

	// Get the value
	RegQueryValueEx(key, setting, NULL, NULL, (BYTE *) value.GetBuffer(size), &size);
	value.ReleaseBuffer();

	return value;
}

void CZopeDialog::OnCancel() 
{
	// The Cancel button is actually marked "Close"
	
	Cleanup();
	
	CDialog::OnCancel();
}

void CZopeDialog::Cleanup()
{
	// Prompt to save changes first
	if (m_Apply.IsWindowEnabled())
	{
		int nResult = MessageBox("You have made some changes to this configuration since you last saved.  Would you like to apply your changes before leaving?", theApp.m_DialogCaption, MB_ICONQUESTION | MB_YESNO);
		if (nResult == IDYES)
			OnOK();
	}
	
	// Save all settings we are going down	
	HKEY hZopeKey;
	DWORD disposition;	// Ignored
	RegCreateKeyEx(HKEY_LOCAL_MACHINE, theApp.regKey, NULL, "", NULL, KEY_WRITE, NULL, &hZopeKey, &disposition);


	// Save current service as default for next time
	int sel = m_ConfigurationList.GetCurSel();
	if (sel != CB_ERR)
	{
		ZopeService *service = (ZopeService *) m_ConfigurationList.GetItemDataPtr(sel);		
		RegSetValueEx(hZopeKey, "DefaultService", NULL, REG_SZ, (BYTE *) (LPCSTR) service->m_ServiceName, service->m_ServiceName.GetLength());
	}

	// Save window position
	RECT pos;
	GetWindowRect(&pos);
	RegSetValueEx(hZopeKey, "WindowX", NULL, REG_DWORD, (BYTE *) &pos.left, sizeof(pos.left));
	RegSetValueEx(hZopeKey, "WindowY", NULL, REG_DWORD, (BYTE *) &pos.top, sizeof(pos.top));
 

	RegCloseKey(hZopeKey);

	// Clean up
	for (int i = 0; i < m_ConfigurationList.GetCount(); i++)
		delete (ZopeService *) m_ConfigurationList.GetItemDataPtr(i);

	if (schSCManager)
		CloseServiceHandle(schSCManager);
}

void CZopeDialog::OnOK() 
{
	// The OK button is actually marked "Apply"	
	
	ApplyChanges();
}

void CZopeDialog::OnDestroy() 
{
	CDialog::OnDestroy();

	if (propertySheet)
		propertySheet->DestroyWindow();

	delete servicesPage;
	delete logPage;
	delete aboutPage;
	delete emergencyUserPage;
	delete executionPage;
	delete controlPage;
	delete propertySheet;

}

void CZopeDialog::ShowHelpTab()
{
	if (servicesPage)
	{
		// Remove all the configuration tabs
		propertySheet->RemovePage(servicesPage);
		propertySheet->RemovePage(logPage);
		propertySheet->RemovePage(aboutPage);
		propertySheet->RemovePage(executionPage);
		propertySheet->RemovePage(emergencyUserPage);
		propertySheet->RemovePage(controlPage);
		delete executionPage;
		delete servicesPage;
		delete logPage;
		delete emergencyUserPage;
		delete aboutPage;
		delete controlPage;
		servicesPage = NULL;
		logPage = NULL;
		aboutPage = NULL;
		executionPage = NULL;
		emergencyUserPage = NULL;
		controlPage = NULL;
	}

	if (!aboutPage)
	{
		// Show the help tab
		aboutPage = new CAboutPage();
		aboutPage->simpleHelp = TRUE;
		aboutPage->zopeDialog = this;
		propertySheet->AddPage(aboutPage);
		propertySheet->SetActivePage(0);
		
	}
}

void CZopeDialog::ShowAllTabs()
{
	if (!servicesPage)
	{
		if (aboutPage)
		{
			propertySheet->RemovePage(aboutPage);
			delete aboutPage;
			aboutPage = NULL;
		}

		// Construct pages
		servicesPage = new CServicesPage();
		logPage = new CLogPage();
		aboutPage = new CAboutPage();
		executionPage = new CExecutionPage();
		emergencyUserPage = new CEmergencyUserPage();
		controlPage = new CControlPage();
		servicesPage->zopeDialog = this;
		logPage->zopeDialog = this;
		executionPage->zopeDialog = this;
		aboutPage->zopeDialog = this;
		emergencyUserPage->zopeDialog = this;
		controlPage->zopeDialog = this;

		// Add pages to tab control
		propertySheet->AddPage(controlPage);
		propertySheet->AddPage(servicesPage);
		propertySheet->AddPage(executionPage);
		propertySheet->AddPage(emergencyUserPage);
		propertySheet->AddPage(logPage);
		propertySheet->AddPage(aboutPage);
		
		// Show first page as default
		propertySheet->SetActivePage(0);	

	}
}


void CZopeDialog::OnAdvanced() 
{
	CAdvancedDialog dlg;
	dlg.m_NTDisabled = !m_ServiceManagerAvailable;
	dlg.m_DaemonNT = m_UseNTServiceDaemon;
	dlg.m_PathNT = m_DaemonNTPath;
	dlg.m_Path95 = m_Daemon95Path;
	dlg.m_PythonHome = m_GlobalPythonHome;
	dlg.m_SoftwareHome = m_GlobalSoftwareHome;
	dlg.m_StartupScript = m_GlobalStartupScript;
	dlg.m_Interpreter = m_GlobalInterpreter;
	
	int nResult = dlg.DoModal();

	// TODO:  Add a "don't tell me this again" check
	if (nResult == IDOK)
		
	{
		// Update new values
		m_UseNTServiceDaemon = dlg.m_DaemonNT;
		m_DaemonNTPath = dlg.m_PathNT;
		m_Daemon95Path = dlg.m_Path95;
		m_GlobalPythonHome = dlg.m_PythonHome;
		m_GlobalSoftwareHome = dlg.m_SoftwareHome;
		m_GlobalStartupScript = dlg.m_StartupScript;
		m_GlobalInterpreter = dlg.m_Interpreter;

		// Save to registry
		HKEY hZopeKey;
		DWORD disposition;	// Ignored
			
		RegCreateKeyEx(HKEY_LOCAL_MACHINE, theApp.regKey, NULL, "", NULL, KEY_WRITE, NULL, &hZopeKey, &disposition);
		RegSetValueEx(hZopeKey, "Daemon95Path", NULL, REG_SZ, (BYTE *) (LPCSTR) m_Daemon95Path, m_Daemon95Path.GetLength());
		RegSetValueEx(hZopeKey, "DaemonNTPath", NULL, REG_SZ, (BYTE *) (LPCSTR) m_DaemonNTPath, m_DaemonNTPath.GetLength());
		RegSetValueEx(hZopeKey, "UseNTServiceDaemon", NULL, REG_DWORD, (BYTE *) &m_UseNTServiceDaemon, sizeof(m_UseNTServiceDaemon));
		RegSetValueEx(hZopeKey, "SoftwareHome", NULL, REG_SZ, (BYTE *) (LPCSTR) m_GlobalSoftwareHome, m_GlobalSoftwareHome.GetLength());
		RegSetValueEx(hZopeKey, "PythonHome", NULL, REG_SZ, (BYTE *) (LPCSTR) m_GlobalPythonHome, m_GlobalPythonHome.GetLength());
		RegSetValueEx(hZopeKey, "Interpreter", NULL, REG_SZ, (BYTE *) (LPCSTR) m_GlobalInterpreter, m_GlobalInterpreter.GetLength());
		RegSetValueEx(hZopeKey, "StartupScript", NULL, REG_SZ, (BYTE *) (LPCSTR) m_GlobalStartupScript, m_GlobalStartupScript.GetLength());
		RegCloseKey(hZopeKey);

		// Delete all entries from the service manager
		// (new ones will be created automatically 
		//  when the services are saved)
		for (int i = 0; i < m_ConfigurationList.GetCount(); i++)
		{
			ZopeService *service = (ZopeService *) m_ConfigurationList.GetItemDataPtr(i);
			service->RemoveServiceEntry();
		}
		
		// Open or close service manager if required
		if (m_UseNTServiceDaemon && !schSCManager)
		{
			// We need to open a handle to the service manager
			schSCManager = OpenSCManager(NULL, NULL, SC_MANAGER_ALL_ACCESS); 
		}
		else if (!m_UseNTServiceDaemon && schSCManager)
		{
			// Close the service manager handle
			CloseServiceHandle(schSCManager);
			schSCManager = NULL;
		}

		// Click the apply button -- we need to do this
		// to propogate the default service daemon
		// onto existing configurations.

		// (This may have some unwanted side-effects)
		ApplyChanges();


	}

}

BOOL CZopeDialog::ServiceExists(CString name)
{
	// If it's in the list box	
	if (m_ConfigurationList.FindStringExact(-1, name) != CB_ERR)
		return TRUE;

	// If it's installed as a service (perhaps 
	// marked for deletion?)
	if (schSCManager)
	{
		SC_HANDLE service = OpenService(schSCManager, name, STANDARD_RIGHTS_REQUIRED);
		if (service)
		{
			CloseServiceHandle(service);
			return TRUE;
		}
	}

	return FALSE;
}

void CZopeDialog::ApplyChanges()
{
		// This takes a few seconds on slow computers so 
	// show an hourglass
	BeginWaitCursor();
	
	// Disable Apply button
	m_Apply.EnableWindow(FALSE);

	// Copy current values into ZopeService class
	ASSERT(propertySheet->GetActivePage());
	propertySheet->GetActivePage()->OnKillActive();

	// Save all configurations to registry
	for (int i = 0; i < m_ConfigurationList.GetCount(); i++)
	{
		ZopeService *service = (ZopeService *) m_ConfigurationList.GetItemDataPtr(i);
		service->schSCManager = schSCManager;
		if (m_UseNTServiceDaemon)
			// NOTE: the path is quoted as it may contain spaces
			service->Save("\"" + m_DaemonNTPath + "\"");	
		else
			// NOTE: the path is quoted as it may contain spaces
			service->Save("\"" + m_Daemon95Path + "\"");

		service->UpdateDefaults(m_GlobalInterpreter,
								m_GlobalPythonHome,
								m_GlobalSoftwareHome,
								m_GlobalStartupScript);
	}

	// Refresh display of current tab
	propertySheet->GetActivePage()->OnSetActive();

	
	// Back to normal mouse cursor
	EndWaitCursor();
}


CString CZopeDialog::AddTrailingSlash(CString path)
{
	// This really should be in CString, yes??
	return path + (path.Right(1) == "\\" ? "" : "\\");
}

void CZopeDialog::AddTrailingSlash(CString *path)
{
	*path += (path->Right(1) == "\\" ? "" : "\\");
}

void CZopeDialog::DoSetupWizard()
{
	// This proc called ONLY after the end of the setup script
	// to create an initial instance or whatever
	m_SetupWizard = TRUE;

	HKEY hZopeKey;
	DWORD disposition;	// ignored
	char CurrentDir[MAX_PATH];
	GetCurrentDirectory(MAX_PATH, CurrentDir);
	RegCreateKeyEx(HKEY_LOCAL_MACHINE, theApp.regKey, NULL, "", NULL, KEY_READ | KEY_WRITE, NULL, &hZopeKey, &disposition);
		
	// Have a look for some stuff that the installer might have given us
	m_DefaultInstanceDir = GetSetting(hZopeKey, "DefaultInstanceDir", CurrentDir);
	m_GlobalSoftwareHome = GetSetting(hZopeKey, "SoftwareHome", "");
	m_GlobalStartupScript = GetSetting(hZopeKey, "StartupScript", "");
	m_GlobalInterpreter = GetSetting(hZopeKey, "Interpreter", ""); 
	m_GlobalPythonHome = GetSetting(hZopeKey, "PythonHome", "");
	
	CPropertySheet wizard(IDS_CAPTION_WIZARD, this);
	CAddName pageName;
	CAddSource pageSource;
	CAddInstanceDir pageInstance;
	CAddInterpreter pageInterpreter;
	CAddSoftware pageSoftware;
	CAddManager pageManager;
	CAddFinish pageFinish;

	pageName.zopeDialog = this;
	pageSource.zopeDialog = this;
	pageInstance.zopeDialog = this;
	pageInterpreter.zopeDialog = this;
	pageSoftware.zopeDialog = this;
	pageManager.zopeDialog = this;
	pageFinish.zopeDialog = this;

	wizard.AddPage(&pageName);
	wizard.AddPage(&pageSource);
	wizard.AddPage(&pageInstance);
	wizard.AddPage(&pageInterpreter);
	wizard.AddPage(&pageSoftware);
	wizard.AddPage(&pageManager);
	wizard.AddPage(&pageFinish);
	wizard.SetWizardMode();
	int nResult = wizard.DoModal();
  
	if (nResult == ID_WIZFINISH)
	{

		// Check for creation of global defaults
		if (m_GlobalInterpreter == "")
		{
			if (pageInstance.m_InterpreterInInstance)
				m_GlobalInterpreter = AddTrailingSlash(pageInstance.m_Path) + "python.exe";
			else
				m_GlobalInterpreter = pageInterpreter.m_Interpreter;
		}
		
		if (m_GlobalPythonHome == "")
		{
			// Extrapolate PythonHome
			m_GlobalPythonHome = m_GlobalInterpreter.Left(m_GlobalInterpreter.ReverseFind('\\'));
		}

		if (m_GlobalStartupScript == "")
		{
			if (pageInstance.m_StartupScriptInInstance)
			{
				CString script;
				CFileStatus status;
				if (CFile::GetStatus(AddTrailingSlash(pageInstance.m_Path) + "z2.py", status))
					script = "z2.py";
				else
					script = "z2s.py";
				m_GlobalStartupScript = AddTrailingSlash(pageInstance.m_Path) + script;
			}
			else
			{
				m_GlobalStartupScript = pageSoftware.m_StartupScript;
			}
		}

		if (m_GlobalSoftwareHome == "")
		{
			// Extrapolate
			m_GlobalSoftwareHome = m_GlobalStartupScript.Left(m_GlobalStartupScript.ReverseFind('\\'));
		}
		
		// Save global vars
		RegSetValueEx(hZopeKey, "SoftwareHome", NULL, REG_SZ, (BYTE *) (LPCSTR) m_GlobalSoftwareHome, m_GlobalSoftwareHome.GetLength() + 1);
		RegSetValueEx(hZopeKey, "PythonHome", NULL, REG_SZ, (BYTE *) (LPCSTR) m_GlobalPythonHome, m_GlobalPythonHome.GetLength() + 1);
		RegSetValueEx(hZopeKey, "StartupScript", NULL, REG_SZ, (BYTE *) (LPCSTR) m_GlobalStartupScript, m_GlobalStartupScript.GetLength() + 1);
		RegSetValueEx(hZopeKey, "Interpreter", NULL, REG_SZ, (BYTE *) (LPCSTR) m_GlobalInterpreter, m_GlobalInterpreter.GetLength() + 1);
		if (m_DefaultInstanceDir == CurrentDir)
			RegSetValueEx(hZopeKey, "DefaultInstanceDir", NULL, REG_SZ, (BYTE *) (LPCSTR) m_GlobalSoftwareHome, m_GlobalSoftwareHome.GetLength() + 1);

		// Detect NT or 95 Services
		schSCManager = OpenSCManager(NULL, NULL, SC_MANAGER_ALL_ACCESS);
		m_UseNTServiceDaemon = schSCManager ? TRUE : FALSE;
		
		// Create a new service
		ZopeService *newService = new ZopeService(theApp.servicesKey, 
												  pageName.m_Name, 
												  schSCManager, 
												  m_GlobalSoftwareHome,
												  m_GlobalStartupScript,
												  m_GlobalPythonHome,
												  m_GlobalInterpreter);

		newService->m_InstanceHome = pageInstance.m_Path;
		newService->m_ClientHome = AddTrailingSlash(pageInstance.m_Path) + "var";
		
		
		if (pageSource.m_SourceType == CreateNew)
		{
			// Create the instance dir and a Var dir beneath it
			CreateDirectory(newService->m_InstanceHome, NULL);
			CreateDirectory(newService->m_ClientHome, NULL);
		}

		if (pageSource.m_SourceType == CreateNew)
		{
			// Create initial manager
			FILE *fInitUser = fopen(AddTrailingSlash(newService->m_InstanceHome) +  "inituser", "wt");
			if (!fInitUser)
				MessageBox("Warning: Unable to create initial manager.", theApp.m_DialogCaption, MB_ICONEXCLAMATION);
			else
			{
				fprintf(fInitUser, "%s:%s", pageManager.m_Username, pageManager.m_Password1);
				fclose(fInitUser);
			}
		}

		// Save it to registry
		BOOL bResult;
		if (m_UseNTServiceDaemon)
			bResult = newService->Save("\"" + m_DaemonNTPath + "\"");
		else
			bResult = newService->Save("\"" + m_Daemon95Path + "\"");

		
		if (!bResult)
		{
			// Something went wrong TODO error checking
			MessageBox("The service could not be created.  It is possible that a service with this name already exists, or has been marked for deletion.  You may want to try restarting your computer to fix the problem, or renaming the service.", theApp.m_DialogCaption, MB_ICONEXCLAMATION);
			newService->Remove();
			delete newService;
			return;
		}
	}

	RegCloseKey(hZopeKey);


}