
#include "servicecommon.h"
#include "NTServiceInfo.h"

#include <direct.h>

// Config class.. 
ZopeService::ZopeService(LPCSTR strParentKey,
						 LPCSTR strService,
						 SC_HANDLE v_schSCManager)
: schSCManager(v_schSCManager),
  m_ServiceName(strService),
  m_UseGlobalDefaults(TRUE)
{
	// A shortcut for the services which don't like to deal with
	// the registry

	CString strKey = AddTrailingSlash(strParentKey) + "services";
	m_ServicesKey = strKey;

	HKEY hZopeKey;
	int success = RegOpenKeyEx(HKEY_LOCAL_MACHINE, strParentKey, NULL, KEY_READ, &hZopeKey);

	char CurrentDir[MAX_PATH];
	GetCurrentDirectory(MAX_PATH, CurrentDir);
	AddTrailingSlash(CurrentDir);
	CString strSoftwareHome = GetSetting(hZopeKey, "SoftwareHome", CurrentDir);
	CString strStartupScript = GetSetting(hZopeKey, "StartupScript", AddTrailingSlash(strSoftwareHome) + "z2.py");
	CString strPythonHome = GetSetting(hZopeKey, "PythonHome", CurrentDir);
	CString strInterpreter = GetSetting(hZopeKey, "Interpreter", AddTrailingSlash(strPythonHome) + "python.exe");

	if (success == ERROR_SUCCESS)
		RegCloseKey(hZopeKey);

	Init(strSoftwareHome,
		 strStartupScript,
		 strPythonHome,
		 strInterpreter);
}


ZopeService::ZopeService(LPCSTR strKey, 
						 LPCSTR strService, 
						 SC_HANDLE v_schSCManager, 
						 LPCSTR strSoftwareHome,
						 LPCSTR strStartupScript,
						 LPCSTR strPythonHome,
						 LPCSTR strInterpreter)
: schSCManager(v_schSCManager),
  m_ServicesKey(strKey),
  m_ServiceName(strService),
  m_UseGlobalDefaults(TRUE)
{
	/*	strKey is the reg path to the services key
		from HKEY_LOCAL_MACHINE.  typically:
	
			"Software\\Zope\\Services"
	
		strService is the service name, e.g.:

			"WebSite"   or  "BizarShop"

		schSCManager is a handle to the NT service manager
		if it is being used or NULL otherwise (use 95 services)
	*/

	Init(strSoftwareHome,
		 strStartupScript,
		 strPythonHome,
		 strInterpreter);
}
	
ZopeService::Init(LPCSTR strSoftwareHome,
				  LPCSTR strStartupScript,
				  LPCSTR strPythonHome,
				  LPCSTR strInterpreter)
{
	HKEY hKeyService;
	int success;

	// Open key
	AddTrailingSlash(&m_ServicesKey);
	strServiceKey = m_ServicesKey + m_ServiceName;
	success = RegOpenKeyEx(HKEY_LOCAL_MACHINE, strServiceKey, NULL, KEY_READ, &hKeyService);

	// Load values or defaults
	m_InstanceHome = GetSetting(hKeyService, "InstanceHome", "");
	AddTrailingSlash(m_InstanceHome);

	m_SoftwareHome = GetSetting(hKeyService, "SoftwareHome", "DEFAULT");
						// Typically this is set globally however a particular
						// instance can override if it wants to
	if (m_SoftwareHome == "DEFAULT")
		m_SoftwareHome = strSoftwareHome;
	AddTrailingSlash(m_SoftwareHome);

	m_ClientHome  = GetSetting(hKeyService, "ClientHome", "DEFAULT");
	if (m_ClientHome == "DEFAULT")
		m_ClientHome = m_InstanceHome + "var\\";
	AddTrailingSlash(m_ClientHome);

	m_PythonHome = GetSetting(hKeyService, "PythonHome", "DEFAULT");
						// Another variable which is typically global
						// but you might want to specify individually (eg. debugging)
	if (m_PythonHome == "DEFAULT")
		m_PythonHome = strPythonHome;
	else
		m_UseGlobalDefaults = FALSE;
	AddTrailingSlash(m_PythonHome);

	m_Interpreter = GetSetting(hKeyService, "Interpreter", "DEFAULT");
	if (m_Interpreter == "DEFAULT")
		m_Interpreter = strInterpreter;
	else
		m_UseGlobalDefaults = FALSE;

	m_StartupScript = GetSetting(hKeyService, "StartupScript", "DEFAULT");
	if (m_StartupScript == "DEFAULT")
		m_StartupScript = strStartupScript;
	else
		m_UseGlobalDefaults = FALSE;

	m_StdOutLogFile = GetSetting(hKeyService, "StdOutLogFile", "");
	m_StupidLogFile = GetSetting(hKeyService, "StupidLogFile", "");
	
	if (_stricmp(CString(strStartupScript).Right(6), "z2s.py") == 0)
	{
		// Allow secure ports by default
		strcpy(https, GetSetting(hKeyService, "HTTPS", "8443"));
		strcpy(webdavs, GetSetting(hKeyService, "WEBDAVS", "8444"));
	}
	else
	{
		strcpy(https, GetSetting(hKeyService, "HTTPS", ""));
		strcpy(webdavs, GetSetting(hKeyService, "WEBDAVS", ""));
	}
	strcpy(http, GetSetting(hKeyService, "HTTP", "8080"));
	strcpy(ftp, GetSetting(hKeyService, "FTP", "8021"));
	strcpy(webdav, GetSetting(hKeyService, "WEBDAV", "8081"));
	
	m_ShowTrayIcon = GetSetting(hKeyService, "ShowTrayIcon", (DWORD) TRUE);

	// Finished reading from this part of the registry
	if (success == ERROR_SUCCESS)
		RegCloseKey(hKeyService);	

	// Find out if service starts up with system
	if (schSCManager)
	{
		// Query the NT service manager
		SC_HANDLE service = OpenService(schSCManager, m_ServiceName, SERVICE_QUERY_CONFIG);
		if (service)
		{
			NTServiceInfo serviceConfig(service);
			m_SystemStartup = (serviceConfig.config->dwStartType == SERVICE_AUTO_START);
			CloseServiceHandle(service);
		} 
		else 
		{
			// This is clearly an error, but ignore it, it's already reported
			m_SystemStartup = FALSE;
		}

	} 
	else
	{
		// Look it up in the registry (win 95)
		HKEY hRunServices;
		DWORD type; // ignored
		DWORD size; // ignored
		char buffer; // ignored

		int success = RegOpenKeyEx(HKEY_LOCAL_MACHINE, "Software\\Microsoft\\Windows\\CurrentVersion\\RunServices", NULL, KEY_QUERY_VALUE, &hRunServices);
		if (success == ERROR_SUCCESS)
		{
			success = RegQueryValueEx(hRunServices,
										  m_ServiceName,
										  NULL,
										  &type,
										  (BYTE *) &buffer,
										  &size);
			m_SystemStartup = (success != ERROR_FILE_NOT_FOUND);
			RegCloseKey(hRunServices);
		}
	}


	
	// Extrapolate location of PID file
	strZopePIDFile = AddTrailingSlash(m_ClientHome) + "z2.pid";

	// Init sockets
	WORD wVersion = MAKEWORD(1, 1);
	WSADATA wsadata;
	WSAStartup(wVersion, &wsadata);	
}


ZopeService::~ZopeService()
{
	// Finalize sockets
	WSACleanup();
}

void ZopeService::Remove()
{
	// Remove this service from registry
	RegDeleteKey(HKEY_LOCAL_MACHINE, strServiceKey);

	// Remove from service list
	RemoveServiceEntry();
}

void ZopeService::RemoveServiceEntry()
{
	if (schSCManager)
	{
		// Remove from NT services list
		SC_HANDLE schService = OpenService(schSCManager, m_ServiceName, DELETE);
		if (schService)
		{
			DeleteService(schService);
			CloseServiceHandle(schService);
		}
	}
	else
	{
		// Remove from Win95 startup list
		HKEY hRunServices;
		DWORD disposition;	// Ignored
		RegCreateKeyEx(HKEY_LOCAL_MACHINE,
					   "Software\\Microsoft\\Windows\\CurrentVersion\\RunServices",
					   NULL,
					   "",
					   REG_OPTION_VOLATILE,
					   KEY_SET_VALUE,
					   NULL,
					   &hRunServices,
					   &disposition);
		
		// Remove the key
		RegDeleteValue(hRunServices, m_ServiceName);
		RegCloseKey(hRunServices);
	}
}

BOOL ZopeService::Save(CString serviceDaemon)
{
	// serviceDaemon is the full path to either ZopeServiceNT.exe or ZopeService95.exe
	// This is needed so we can fill in the startup parameters

	// TODO: some error handling

	HKEY hKeyService;
	
	// This shouldn't be changed but just in case...
	AddTrailingSlash(m_ServicesKey);	
	strServiceKey = m_ServicesKey + m_ServiceName;
		
	// Save current state to the registry
	DWORD disposition;
	RegCreateKeyEx(HKEY_LOCAL_MACHINE, strServiceKey, NULL, "", REG_OPTION_NON_VOLATILE, KEY_WRITE, NULL, &hKeyService, &disposition);
	
	SaveSetting(hKeyService, "InstanceHome", m_InstanceHome);
	SaveSetting(hKeyService, "ClientHome", m_ClientHome);
	if (!m_UseGlobalDefaults)
	{	
		SaveSetting(hKeyService, "PythonHome", m_PythonHome);
		SaveSetting(hKeyService, "SoftwareHome", m_SoftwareHome);
		SaveSetting(hKeyService, "StartupScript", m_StartupScript);
		SaveSetting(hKeyService, "Interpreter", m_Interpreter);
	}
	else
	{
		RegDeleteValue(hKeyService, "PythonHome");
		RegDeleteValue(hKeyService, "SoftwareHome");
		RegDeleteValue(hKeyService, "StartupScript");
		RegDeleteValue(hKeyService, "Interpreter");
	}

	
	// Ports
	SaveSetting(hKeyService, "HTTP", http);
	SaveSetting(hKeyService, "HTTPS", https);
	SaveSetting(hKeyService, "FTP", ftp);
	SaveSetting(hKeyService, "WEBDAV", webdav);
	SaveSetting(hKeyService, "WEBDAVS", webdavs);

	// Logs
	SaveSetting(hKeyService, "StdOutLogFile", m_StdOutLogFile);
	SaveSetting(hKeyService, "StupidLogFile", m_StupidLogFile);

	// Show in tray 
	SaveSetting(hKeyService, "ShowTrayIcon", m_ShowTrayIcon);

	RegCloseKey(hKeyService);

	// If we are using NT services then update the
	// service manager entry
	if (schSCManager)
	{
		SC_HANDLE schService = OpenService(schSCManager, m_ServiceName, SERVICE_CHANGE_CONFIG | SERVICE_QUERY_CONFIG);
		if (!schService)
		{
			// We need to create the service entry
			CString cmd = serviceDaemon + " \"" + m_ServiceName + "\"";
			schService = CreateService(schSCManager,
									   m_ServiceName,	// Service name
									   "Zope (" + m_ServiceName + ")",	// Display name
									   SERVICE_ALL_ACCESS,			// Desired access
									   SERVICE_WIN32_OWN_PROCESS,	// Service type
									   SERVICE_DEMAND_START,		// Start type
									   SERVICE_ERROR_NORMAL,		// Error Control
									   cmd,				// Binary path name
									   NULL,						// Load order group
									   NULL,						// Tag ID
									   NULL,						// Dependencies
									   NULL,						// User account
									   NULL						// Password
									  );
		}
		if (!schService)
		{
			return FALSE;
		}

		NTServiceInfo serviceInfo(schService);
		if (m_SystemStartup)
			serviceInfo.changeStartupType(SERVICE_AUTO_START);
		else
			serviceInfo.changeStartupType(SERVICE_DEMAND_START);
		CloseServiceHandle(schService);
	}
	else
	{
		// We are running Windows 95/98
		// Use the registry keys to set/unset system
		// startup

		HKEY hRunServices;
		DWORD disposition;	// Ignored
		RegCreateKeyEx(HKEY_LOCAL_MACHINE,
					   "Software\\Microsoft\\Windows\\CurrentVersion\\RunServices",
					   NULL,
					   "",
					   REG_OPTION_VOLATILE,
					   KEY_SET_VALUE,
					   NULL,
					   &hRunServices,
					   &disposition);
		
		if (m_SystemStartup)
		{
			// Add the key
			CString cmd = "\"" + serviceDaemon + "\" \"" + m_ServiceName + "\"";
			RegSetValueEx(hRunServices, m_ServiceName, NULL, REG_SZ, (BYTE *) (LPCSTR) cmd, cmd.GetLength());
		}
		else
		{
			// Remove the key
			RegDeleteValue(hRunServices, m_ServiceName);
		}
		if (hRunServices)
			RegCloseKey(hRunServices);

	}
					 
	
	return TRUE;
}



void ZopeService::SaveSetting(HKEY key, LPCSTR setting, LPCSTR value)
{
	RegSetValueEx(key, setting, NULL, REG_SZ, (BYTE *) value, strlen(value) + 1);
}

void ZopeService::SaveSetting(HKEY key, LPCSTR setting, DWORD value)
{
	RegSetValueEx(key, setting, NULL, REG_DWORD, (BYTE *) &value, sizeof(value));
}
	

CString ZopeService::GetSetting(HKEY key, LPCSTR setting, CString defaultSetting)
{
	// This function reads a value tied to <setting> from the registry in <key>.
	// If it doesn't exist return <defaultSetting>

	LONG success;
	DWORD size;
	CString value;

	// Get the size of the value
	success = RegQueryValueEx(key, setting, NULL, NULL, NULL, &size);

	if (success != ERROR_SUCCESS)
		return defaultSetting;

	// Get the value
	RegQueryValueEx(key, setting, NULL, NULL, (BYTE *) value.GetBuffer(size), &size);
	value.ReleaseBuffer();

	return value;
}


DWORD ZopeService::GetSetting(HKEY key, LPCSTR setting, DWORD defaultSetting)
{
	// This function reads a value tied to <setting> from the registry in <key>.
	// If it doesn't exist return <defaultSetting>

	LONG success;
	DWORD value;
	DWORD size = sizeof(value);
	
	// Get the value
	success = RegQueryValueEx(key, setting, NULL, NULL, (BYTE *) &value, &size);

	if (success != ERROR_SUCCESS)
		return defaultSetting;


	return value;
}

long ZopeService::GetZopeProcessID()
{
	// Does not guarantee the process is still running.. 
	// use QueryStatus() instead
	
	// Open pid file
	FILE *pidFile = fopen(strZopePIDFile, "r");
	if (!pidFile)
	{
		// PID file does not exist
		return NULL;
	}

	// Read the first number from pidFile
	char buf[10];
	fread(buf, 1, 10, pidFile);
	fclose(pidFile);
	return strtol(buf, NULL, 10);
}

HANDLE ZopeService::GetZopeProcessHandle()
{
	/* You must close the returned handle from
	   GetZopeProcessHandle() */

	long ZopePID = GetZopeProcessID();
	
	// Try to open a handle to the process
	return OpenProcess(PROCESS_TERMINATE, FALSE, ZopePID);

}

DWORD ZopeService::QueryStatus()
{
	// Return either SERVICE_RUNNING or SERVICE_STOPPED

	HANDLE hProc = GetZopeProcessHandle();
	if (hProc)
	{
		// Process is running
		CloseHandle(hProc);
		
		if (ZopeListening())
			return SERVICE_RUNNING;
		else
		{
			// Zope is running and has written a PID file
			// but obviously hasn't opened its ports yet
			return SERVICE_START_PENDING;
		}

	}
	else
	{
		// It looks like it's not running, but maybe it just
		// hasn't had a chance to startup yet?

		// Check if we started in the last 15 seconds..
		if (time(NULL) - StartTime < 15)
		{
			// We did.. not likely that the PID file has
			// been written yet
			return SERVICE_START_PENDING;
		}
		else
		{
			// Process is definitely not running
			return SERVICE_STOPPED;
		}
	}
}

BOOL ZopeService::StartZope()
{
	// Start the Zope process
	
	if (QueryStatus() == SERVICE_RUNNING)
		return TRUE;

	// Kill the PID file
	// This removes any uncertainty about the PID being reused by the OS
	// (perhaps after restarting) for another app.
	DeleteFile(strZopePIDFile);

	// Kill Stupid log file
	DeleteFile(m_StupidLogFile);

	// Kill Stdout log file
	DeleteFile(m_StdOutLogFile);

	
	_chdir(m_SoftwareHome);

	// Win32 gunk to get the process going
	PROCESS_INFORMATION ps_info;
	STARTUPINFO s_info;
	ZeroMemory(&s_info, sizeof(s_info));
	s_info.cb = sizeof(s_info);
	s_info.dwFlags = STARTF_USESHOWWINDOW;
	s_info.wShowWindow = SW_HIDE;


	int nResult = 0;

	// Add ports to command line
	CString cmdLine = "\"" + m_Interpreter + "\" \"" + m_StartupScript + "\"";
	cmdLine += " -X";	// Clear all ports before starting

	if (http[0])
		cmdLine += CString(" -w") + http;

	if (https[0])
		cmdLine += CString(" -y") + https;

	if (ftp[0])
		cmdLine += CString(" -f") + ftp;

	if (webdav[0])
		cmdLine += CString(" -W") + webdav;

	if (webdavs[0])
		cmdLine += CString(" -Y") + webdavs;

/* Instead of creating a new env block we just add the variables
   to the existing one and make a copy of it. (otherwise winsock fails)

	// Create environment block
	char envBlock[MAX_PATH * 5 + 100]; // Plenty of room for 5 paths
	char *pEnvBlock = envBlock;

	// Working directory
	char currentDir[MAX_PATH];
	GetCurrentDirectory(MAX_PATH, currentDir);
	if (currentDir[0] != '\\') 
	{
		// Not a UNC path, so add it to the env block
		strcpy(pEnvBlock, "=");
		strncat(pEnvBlock, currentDir, 1);
		strcat(pEnvBlock, ":=");
		strcat(pEnvBlock, currentDir);
		pEnvBlock += strlen(pEnvBlock) + 1;
	}
	
	// CLIENT_HOME
	strcpy(pEnvBlock, "CLIENT_HOME=");
	strcat(pEnvBlock, m_ClientHome);
	pEnvBlock += strlen(pEnvBlock) + 1;

	// INSTANCE_HOME
	strcpy(pEnvBlock, "INSTANCE_HOME=");
	strcat(pEnvBlock, m_InstanceHome);
	pEnvBlock += strlen(pEnvBlock) + 1;

	// PYTHON_HOME
	strcpy(pEnvBlock, "PYTHONHOME=");
	strcat(pEnvBlock, m_PythonHome);
	pEnvBlock += strlen(pEnvBlock) + 1;

	// SOFTWARE_HOME
	strcpy(pEnvBlock, "SOFTWARE_HOME=");
	strcat(pEnvBlock, m_SoftwareHome);
	pEnvBlock += strlen(pEnvBlock) + 1;

	// STUPID_LOG_FILE
	if (m_StupidLogFile != "")
	{
		strcpy(pEnvBlock, "STUPID_LOG_FILE=");
		strcat(pEnvBlock, m_StupidLogFile);
		pEnvBlock += strlen(pEnvBlock) + 1;
	}

	// Terminating null
	*pEnvBlock = 0;
*/
	SetEnvironmentVariable("INSTANCE_HOME", m_InstanceHome);
	//SetEnvironmentVariable("SOFTWARE_HOME", m_SoftwareHome);
	//SetEnvironmentVariable("PYTHONHOME", m_PythonHome);
	SetEnvironmentVariable("CLIENT_HOME", m_ClientHome);
	if (m_StupidLogFile != "")
		SetEnvironmentVariable("STUPID_LOG_FILE", m_StupidLogFile);

		// Stdout log
	HANDLE hErrorRead = NULL;
	HANDLE hErrorWrite;

	if (m_StdOutLogFile != "")
	{
		// Create a pipe to receive stdout & stderr
		// Security attributes enable handle to be inherited
		SECURITY_ATTRIBUTES sa;
		sa.nLength = sizeof(sa);
		sa.lpSecurityDescriptor = NULL;
		sa.bInheritHandle = TRUE;
		
		HANDLE hErrorReadTmp;
		
		// Create anonymous pipe to retrieve stderr
		// of python
		CreatePipe(&hErrorReadTmp, &hErrorWrite, &sa, 0);
		

		// Create new error read handle to prevent child
		// inheriting properties and creating non-closeable
		// handles
		DuplicateHandle(GetCurrentProcess(), hErrorReadTmp,
							 GetCurrentProcess(),
							 &hErrorRead,	// Address of new handle
							 0, FALSE,		// Un-inheritable
							 DUPLICATE_SAME_ACCESS);

		// Close inheritable copies of the handles
		// you do not want to be inherited
		CloseHandle(hErrorReadTmp);

		// Set the output to redirect
		s_info.hStdInput = GetStdHandle(STD_INPUT_HANDLE);
		s_info.hStdOutput = hErrorWrite;
		s_info.hStdError = hErrorWrite;
		s_info.dwFlags |= STARTF_USESTDHANDLES;
	}


	nResult = CreateProcess(NULL,					// app name
								cmdLine.GetBuffer(cmdLine.GetLength()),
													// command line
								NULL,				// process attributes
								NULL,				// thread attributes
								TRUE,				// inherit handles
								NORMAL_PRIORITY_CLASS,
													// creation flags
								NULL, //envBlock,			// environment block
								m_InstanceHome,		// working dir
								&s_info,			// startup info
								&ps_info);			// process info

	if (nResult == 0)
	{
		//TODO (We can't messagebox because we might be a NT service)
		//MessageBox(NULL, "Could not start interpreter (Python)", "Zope", MB_ICONSTOP);
		return FALSE;
	}

	if (hErrorRead)
	{
		CloseHandle(ps_info.hThread);
		CloseHandle(hErrorWrite);
		
		// Create the log file
		m_LogThread_FileHandle = CreateFile(m_StdOutLogFile,
											GENERIC_WRITE,
											FILE_SHARE_READ | FILE_SHARE_WRITE,
											NULL,	// Security
											OPEN_ALWAYS,
											FILE_ATTRIBUTE_NORMAL | FILE_FLAG_WRITE_THROUGH,
											NULL);

		// Start the thread to read from the pipe
		HANDLE hLogThread;
		DWORD threadId;
		m_LogThread_Run = TRUE;
		m_LogThread_PipeHandle = hErrorRead;

		hLogThread = CreateThread(NULL,
								  0,
								  (LPTHREAD_START_ROUTINE) ThreadLogStdOut,
								  (LPVOID) this,
								  0,
								  &threadId);
		CloseHandle(hLogThread); 

	}


	// Start the timeout for "still starting"
	StartTime = time(NULL);
	return TRUE;
}

DWORD WINAPI ThreadLogStdOut(LPVOID lpParameter)
{
	// Run in a seperate thread to read from the pipe we have
	// created to capture output of python and write it to
	// StdOutLogFile.
	
	ZopeService *service = (ZopeService *) lpParameter;

	SetThreadPriority(GetCurrentThread(), THREAD_PRIORITY_BELOW_NORMAL);


	while (service->m_LogThread_Run)
	{
		char buf[256];
		DWORD nBytesRead = sizeof(buf);

		if (ReadFile(service->m_LogThread_PipeHandle,
				 &buf,
				 nBytesRead,
				 &nBytesRead,
				 NULL))
		{
			if (nBytesRead)
			{
				if (!WriteFile(service->m_LogThread_FileHandle,
							   buf,
							   nBytesRead,
							   &nBytesRead, // Ignored
							   NULL))
					break;
			}
		} 
		else
		{
			break;
		}
			
	}

	// Thread just cleans up and dies if something goes wrong
	// (e.g. other end of pipe is closed)

	CloseHandle(service->m_LogThread_FileHandle);
	CloseHandle(service->m_LogThread_PipeHandle);

	return 0;
}


void ZopeService::StopZope()
{
	HANDLE hProc = GetZopeProcessHandle();
	
	// Kill the process
	if (hProc)
	{
		TerminateProcess(hProc, 0);
		CloseHandle(hProc);
	}


}


BOOL ZopeService::ZopeListening()
{
	/* If we can connect a socket to Zope's www then obviously
	   zope is listening */
	
	// Choose a port to listen to 
	char *checkPort;
	if (http[0])
		checkPort = http;
	else if (https[0])
		checkPort = https;
	else if (ftp[0])
		checkPort = ftp;
	else if (webdav[0])
		checkPort = webdav;
	else if (webdavs[0])
		checkPort = webdavs;
	else
		// Zope isn't even running on any of the ports
		// we know about... lets just skip this test, something
		// else is bound to go wrong anyway.
		return TRUE;

	SOCKADDR_IN addr_in;
	addr_in.sin_family = AF_INET;
	addr_in.sin_addr.S_un.S_addr = 0x0100007F;  // 127.0.0.1
	addr_in.sin_port = htons(atoi(checkPort));	
	
	SOCKET sock;
	sock = socket(AF_INET, SOCK_STREAM, 0);
	if (sock == INVALID_SOCKET)
	{
		// Couldn't create socket!
		return FALSE;
	}
	int con;
	SOCKADDR FAR *addr;
	addr = (SOCKADDR *) &addr_in;
	con = connect(sock, addr, sizeof(addr_in));
	if (con == SOCKET_ERROR)
	{
		// Most likely connection refused
		return FALSE;
	}

	closesocket(sock);

	return TRUE;
}

void ZopeService::UpdateDefaults(LPCSTR strInterpreter,
								 LPCSTR strPythonHome,
								 LPCSTR strSoftwareHome,
								 LPCSTR strStartupScript)
{
	if (m_UseGlobalDefaults)
	{
		m_Interpreter = strInterpreter;
		m_PythonHome = strPythonHome;
		m_SoftwareHome = strSoftwareHome;
		m_StartupScript = strStartupScript;
	}
}

CString ZopeService::AddTrailingSlash(CString path)
{
	// This really should be in CString, yes??
	return path + (path.Right(1) == "\\" ? "" : "\\");
}

void ZopeService::AddTrailingSlash(CString *path)
{
	*path += (path->Right(1) == "\\" ? "" : "\\");
}