// Common to either type of service

#ifndef SERVICE_COMMON_H
#define SERVICE_COMMON_H

#include <afx.h>
#include <winsock2.h>
#include <errno.h>

// Configuration for ONE zope installation
class ZopeService
{
public:
	ZopeService(LPCSTR strKey, 
				LPCSTR strService, 
				SC_HANDLE v_schSCManager, 
				LPCSTR strSoftwareHome,
				LPCSTR strStartupScript,
				LPCSTR strPythonHome,
				LPCSTR strInterpreter);

	ZopeService(LPCSTR strParentKey,
				LPCSTR strService,
				SC_HANDLE);

	virtual ~ZopeService();

private:
	Init(LPCSTR strSoftwareHome,
		 LPCSTR strStartupScript,
		 LPCSTR strPythonHome,
		 LPCSTR strInterpreter);

	// Thread to capture stdout
	friend DWORD WINAPI ThreadLogStdOut(LPVOID lpParameter);
	HANDLE m_LogThread_FileHandle;
	HANDLE m_LogThread_PipeHandle;
	BOOL m_LogThread_Run;


public:
	void UpdateDefaults(LPCSTR strInterpreter,
						LPCSTR strPythonHome,
						LPCSTR strSoftwareHome,
						LPCSTR strStartupScript);

	BOOL Save(CString serviceDaemon);
	void Remove();
	void RemoveServiceEntry();

	static CString AddTrailingSlash(CString path);	//used everywhere
	static void AddTrailingSlash(CString *path);

	/*	These control functions are ONLY to be used
		by the service exe's (ZopeServiceNT and ZopeService95)

		The control panel has its own methods for
		communicating with the services */

	BOOL StartZope();
	void StopZope();
	DWORD QueryStatus();

	// Ports
	char http[6];
	char https[6];
	char ftp[6];
	char webdav[6];				// Kept as strings for simplicity -- they are
	char webdavs[6];			// passed to the command line

	// Registry
	CString m_ServicesKey;		// Registry path to services
	CString m_ServiceName;		// Service name
	
	// Environment / instance variables
	CString m_PythonHome;
	CString m_SoftwareHome;	
	CString m_InstanceHome;
	CString m_ClientHome;		// Defaults to INSTANCE_HOME/var, but included
								// for future compatibility with ZEO

	CString m_Interpreter;		// Full path to interpreter (python.exe)
								// Defaults to PYTHONHOME/python.exe
	CString m_StartupScript;	// Full path to script (z2s.py) or default
								// to SOFTWARE_HOME/z2.py

	// Log files
	CString m_StupidLogFile;	// Full path to STUPID_LOG_FILE 
	CString m_StdOutLogFile;	// Full path to redirect stdout & stderr (optional)
		
	BOOL m_UseGlobalDefaults;	// If FALSE, this service keeps track of its
								// own SOFTWARE_HOME, interpreter, and startup
								// script.
								// If TRUE, use the global values provided
								// to all services

	// Win32 options
	BOOL m_ShowTrayIcon;
	BOOL m_SystemStartup;		// Start up when system starts up

	// NT Services
	SC_HANDLE schSCManager;		// This handle assumed to be open or NULL if
								// we are not using it (use 95 services)


private:	
	void SaveSetting(HKEY key, LPCSTR setting, LPCSTR value);
	CString GetSetting(HKEY key, LPCSTR setting, CString defaultSetting);
	void SaveSetting(HKEY key, LPCSTR setting, DWORD value);
	DWORD GetSetting(HKEY key, LPCSTR setting, DWORD defaultSetting);

	CString strServiceKey;

	long GetZopeProcessID();
	HANDLE GetZopeProcessHandle();
	BOOL ZopeListening();

	time_t StartTime;
	CString strZopePIDFile;
};

DWORD WINAPI ThreadLogStdOut(LPVOID lpParameter);

#endif