// main.cpp

/*
Service95 provides all the functionality that ServiceNT does, but without
utilising any of the Windows NT/2000 service functions.  It also adds a tray
icon to provide an extra place for the user to control it from.  

Interprocess communication with the control panel app is acheived with
windows messages
*/


#include <afx.h>
#include <shellapi.h>
#include <io.h>
#include <fcntl.h>
#include <direct.h>
#include <process.h>

#include "../common/servicecommon.h"

#include "resource.h"

// Global vars

ZopeService *service;
CString serviceName;
NOTIFYICONDATA trayIcon;
HINSTANCE g_hInstance;
PROCESS_INFORMATION ps_info;
HWND wnd;

// Messages
UINT msgZopeStart;
UINT msgZopeStop;
UINT msgZopeQuery;
UINT msgZopeResponse;
UINT msgZopeRegister;
UINT msgZopeUnregister;

// Animation for start pending
const int ANI_TOTAL_FRAMES = 4;
int ANI_CurrentFrame;
const int ANI_FRAMES[4] = {IDI_SERVICE_START_PENDING_1,
						   IDI_SERVICE_START_PENDING_2,
						   IDI_SERVICE_START_PENDING_3,
						   IDI_SERVICE_START_PENDING_4 };

// Registered notification handler (only one permitted)
HWND hwndRegisteredNotificationHandler;


// Forward decls
void UpdateTray(DWORD status);
BOOL DoStart();

#define ZOPE_ICON_NOTIFY 0x0401 // arbitrary number


BOOL CALLBACK wndProc(
  HWND hwndDlg,  // handle to dialog box
  UINT uMsg,     // message
  WPARAM wParam, // first message parameter
  LPARAM lParam  // second message parameter
)
{
	switch (uMsg)
	{
		case WM_INITDIALOG:
		
			return TRUE;

		case WM_TIMER:
		{
			if (wParam == 1)
			{
				// Status poll
				DWORD status = service->QueryStatus();
				UpdateTray(status);
			}
			else if (wParam == 99)
			{
				// Update animation
				ANI_CurrentFrame += 1;
				ANI_CurrentFrame %= ANI_TOTAL_FRAMES;
				trayIcon.hIcon = LoadIcon(g_hInstance, MAKEINTRESOURCE(ANI_FRAMES[ANI_CurrentFrame]));
				Shell_NotifyIcon(NIM_MODIFY, &trayIcon);
			}
		}
		
		case ZOPE_ICON_NOTIFY:
		{
			switch (lParam)
			{
				case WM_RBUTTONUP:
				{
					// Load the popupmenu
					POINT pos;
					GetCursorPos(&pos);

					HMENU mnu = LoadMenu(g_hInstance, MAKEINTRESOURCE(IDR_MENU));
					mnu = GetSubMenu(mnu, 0);

					DWORD status = service->QueryStatus();
					UpdateTray(status);

					if (status == SERVICE_RUNNING) {
						EnableMenuItem(mnu, ID_POPUP_STOP | MF_BYCOMMAND, MF_ENABLED);
						EnableMenuItem(mnu, ID_POPUP_START | MF_BYCOMMAND, MF_GRAYED);
					} else if (status == SERVICE_STOPPED) {
						EnableMenuItem(mnu, ID_POPUP_STOP | MF_BYCOMMAND, MF_GRAYED);
						EnableMenuItem(mnu, ID_POPUP_START | MF_BYCOMMAND, MF_ENABLED);
					} else {
						EnableMenuItem(mnu, ID_POPUP_STOP | MF_BYCOMMAND, MF_GRAYED);
						EnableMenuItem(mnu, ID_POPUP_START | MF_BYCOMMAND, MF_GRAYED);
					}
				
					SetForegroundWindow(hwndDlg);
					TrackPopupMenu(mnu, TPM_RIGHTALIGN | TPM_BOTTOMALIGN, pos.x, pos.y, 0, hwndDlg, NULL);
					PostMessage(hwndDlg, WM_NULL, 0, 0);

					DestroyMenu(mnu);

				}
			}
			return TRUE;
		}

		case WM_COMMAND:
			switch (LOWORD(wParam))
			{
				case ID_POPUP_CLOSE:
				{
					// Are we still running
					DWORD status = service->QueryStatus();
					if (status == SERVICE_STOPPED || MessageBox(NULL, "Closing Zope will stop the web server, and users will no longer be able to view documents.  Are you sure you want to stop Zope?", "Zope", MB_ICONWARNING | MB_OKCANCEL) == IDOK)
					{
						service->StopZope();
						PostQuitMessage(0);
					}
					break;
				}
				case ID_POPUP_START:
					if (DoStart())
					{
						UpdateTray(SERVICE_START_PENDING);
					}
					break;

				case ID_POPUP_STOP:
					service->StopZope();
					UpdateTray(SERVICE_STOPPED);
					break;
				
				case ID_POPUP_CONFIGURE:
					ShellExecute(hwndDlg, NULL, "rundll32.exe", "shell32.dll,Control_RunDLL Zope.cpl", NULL, SW_SHOW);
					break;

/*				case ID_POPUP_ABOUT:
					ShowWindow(hwndDlg, SW_SHOW);
					break;  */

			}
			return true;
		default:
		{
			if (uMsg == msgZopeQuery)
			{
				// The control panel is requesting our status
				// Respond with a message
				
				DWORD status = service->QueryStatus();
				UpdateTray(status);
				SendMessage((HWND) lParam, msgZopeResponse, 0, status);
				return true;
			}
			else if (uMsg == msgZopeStop)
			{
				service->StopZope();
				UpdateTray(SERVICE_STOPPED);
				return true;
			}
			else if (uMsg == msgZopeStart)
			{
				if (DoStart())
				{
					UpdateTray(SERVICE_START_PENDING);
				}
				return true;
			}
			else if (uMsg == msgZopeRegister)
			{
				// Register LPARAM as the notification handler
				hwndRegisteredNotificationHandler = (HWND) lParam;
			}
			else if (uMsg == msgZopeUnregister)
			{
				// Unregister notification handler
				hwndRegisteredNotificationHandler = NULL;
			}
			
		}

	}

	return FALSE;

}


int WINAPI WinMain(
  HINSTANCE hInstance,  // handle to current instance
  HINSTANCE hPrevInstance,  // handle to previous instance
  LPSTR lpCmdLine,      // pointer to command line
  int nCmdShow          // show state of window
)
{

	g_hInstance = hInstance;

	// Extract the configuration name from the command line
	// lpCmdLine contains program arguments excluding the program
	// name.
	serviceName = lpCmdLine;
	
	// Load data from reg
	CString regKey;
	regKey.LoadString(IDS_REGKEY);
	service = new ZopeService(regKey, serviceName, NULL);
	
///////////////////////////////////////////////////////////////////////
	
	// Register Zope-specific window messages
	msgZopeStop = RegisterWindowMessage("MsgZopeStop");
	msgZopeStart = RegisterWindowMessage("MsgZopeStart");
	msgZopeQuery = RegisterWindowMessage("MsgZopeQuery");
	msgZopeResponse = RegisterWindowMessage("MsgZopeResponse");
	msgZopeRegister = RegisterWindowMessage("MsgZopeRegister");
	msgZopeUnregister = RegisterWindowMessage("MsgZopeUnregister");

	// Create a dialog window to receive tray messages but don't show it.
	wnd = CreateDialog(hInstance, MAKEINTRESOURCE(IDD_ABOUT_DIALOG), NULL, wndProc);
	SetWindowText(wnd, CString("ZopeService95") + service->m_ServiceName);	// So that control panel can find us.
	
	// Create tray icon
	if (service->m_ShowTrayIcon)
	{
		HICON icon = LoadIcon(hInstance, MAKEINTRESOURCE(IDI_SERVICE_RUNNING));
	
		trayIcon.cbSize = sizeof(trayIcon);
		trayIcon.hIcon = icon;
		trayIcon.hWnd = wnd;
		strcpy(trayIcon.szTip, "Zope (");
		strcat(trayIcon.szTip, service->m_ServiceName);
		strcat(trayIcon.szTip, ")");
		trayIcon.uCallbackMessage = ZOPE_ICON_NOTIFY;
		trayIcon.uFlags = NIF_ICON | NIF_TIP | NIF_MESSAGE;
		trayIcon.uID = 0;

		// Add tray icon
		Shell_NotifyIcon(NIM_ADD, &trayIcon);
	}

	// Set timer to continually poll Zope for its status (every 5 secs)
	SetTimer(wnd, 1, 5000, NULL);

	
	// Start Zope
	if (DoStart())
		UpdateTray(SERVICE_START_PENDING);
	else
		UpdateTray(SERVICE_STOPPED);

	// Enter the message loop
	MSG msg;
	while (GetMessage(&msg, wnd, NULL, NULL))
	{
		TranslateMessage(&msg);
		DispatchMessage(&msg);
	}

	// Delete tray icon
	if (service->m_ShowTrayIcon)
		Shell_NotifyIcon(NIM_DELETE, &trayIcon);

	// Delete config
	delete service;

	// Kill timer, if any
	KillTimer(wnd, 1);

	return 0;
}

BOOL DoStart()
{
	CString regKey;
	regKey.LoadString(IDS_REGKEY);

	// Reload all values from reg
	delete service;
	service = new ZopeService(regKey, serviceName, NULL);

	return service->StartZope();
}


void UpdateTray(DWORD status)
{

	if (service->m_ShowTrayIcon)
	{
	
		HICON icon;

		if (status == SERVICE_STOPPED)
		{
			// Ensure animation stops
			KillTimer(wnd, 99);
			icon = LoadIcon(g_hInstance, MAKEINTRESOURCE(IDI_SERVICE_STOPPED));
			strcpy(trayIcon.szTip, "Zope is not running");
		}
		else if (status == SERVICE_RUNNING)
		{
			KillTimer(wnd, 99);
			icon = LoadIcon(g_hInstance, MAKEINTRESOURCE(IDI_SERVICE_RUNNING));
			strcpy(trayIcon.szTip, "Zope is running");
		}
		else if (status == SERVICE_START_PENDING)
		{
			icon = LoadIcon(g_hInstance, MAKEINTRESOURCE(ANI_FRAMES[0]));
			ANI_CurrentFrame = 0;
			SetTimer(wnd, 99, 200, NULL);
			strcpy(trayIcon.szTip, "Starting Zope...");
		}
		trayIcon.hIcon = icon;
		Shell_NotifyIcon(NIM_MODIFY, &trayIcon);
	}


	// If there is a registered notification handler,
	// give it our state
	if (hwndRegisteredNotificationHandler)
	{
		SendMessage(hwndRegisteredNotificationHandler, msgZopeResponse, 0, status);
	}

}
