// serviceNT.cpp : Defines the entry point for the console application.
//

#include "stdafx.h"
#include "serviceNT.h"
//#include <iostream.h>

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// The one and only application object

CServiceNTApp theApp;

using namespace std;

void Log(const char *msg, DWORD num = 0)
{
	FILE * log = fopen("error.log", "a");
	fprintf(log, "%d: %s\n", num, msg);
	fclose(log);
}

int _tmain(int argc, TCHAR* argv[], TCHAR* envp[])
{
	
	// initialize MFC 
	if (!AfxWinInit(::GetModuleHandle(NULL), NULL, ::GetCommandLine(), 0))
	{
		cerr << _T("Fatal Error: MFC initialization failed") << endl;
		return 1;
	}

	// String of reg key
	CString regKey;
	regKey.LoadString(IDS_REGKEY);
	
	// Try to create a ZopeService instance for this config
	// given by first argument
	CString serviceName;
	serviceName = argv[1];

	Log(serviceName);
	Log("Creating service");
	
	theApp.service = new ZopeService(regKey, serviceName, NULL);

	// Add handler
	SERVICE_TABLE_ENTRY supportedServices[2];
	supportedServices[0].lpServiceName = _strdup(serviceName);
	supportedServices[0].lpServiceProc = ZopeServiceMain;
	
	// Add terminating null to the list of services
	supportedServices[1].lpServiceName = NULL;
	supportedServices[1].lpServiceProc = NULL;

	Log(argv[1]);
	
	// Start the service controller
	if (!StartServiceCtrlDispatcher(supportedServices))
		Log("Failed StartServiceCtrlDispatcher", GetLastError());

	
	// When we get here the service has stopped
	//delete theApp.service;
	free(supportedServices[0].lpServiceName);

	Log("Finished");

	return 0;

}


// ServiceMain callback
void WINAPI ZopeServiceMain(DWORD argc, char **argv)
{
	Log("ZopeServiceMain");
	Log("Argc", argc);
	Log(argv[0]);
	
	// Register handler
	theApp.statusHandle = RegisterServiceCtrlHandler(argv[0], ZopeServiceHandler);
	if (!theApp.statusHandle)
		Log("RegisterServiceCtrlHandler failed", GetLastError());

	// Give us time...
	SetStatus(SERVICE_START_PENDING);

	
	Log("StartZope");
	Log(theApp.service->m_InstanceHome);

	// Start the service
	if (theApp.service->StartZope())
	{
		SetStatus(SERVICE_START_PENDING);
		
		// Start a monitor thread to make sure it starts up
		// and to notify service manager when it dies
		DWORD threadID; // ignored
		CreateThread(NULL, 0, (LPTHREAD_START_ROUTINE) MonitorThread, theApp.service, 0, &threadID);
	}

	else
	{
		Log("Failed StartZope", GetLastError());
		SetStatus(SERVICE_STOPPED);
	}
	Log("Exit ZopeServiceMain");
}

// Handler callback
void WINAPI ZopeServiceHandler(DWORD control)
{
	switch (control)
	{
	case SERVICE_CONTROL_STOP:
		Log("Received control STOP");
		theApp.service->StopZope();
		SetStatus(SERVICE_STOPPED);
		break;

	case SERVICE_CONTROL_INTERROGATE:
		Log("Received control INTERROGATE");
		SetStatus(theApp.lastState);
		break;

	case SERVICE_CONTROL_SHUTDOWN:
		Log("Received control SHUTDOWN");
		theApp.service->StopZope();
		SetStatus(SERVICE_STOPPED);
		break;

	}
}

void SetStatus(DWORD dwServiceState)
{
	SERVICE_STATUS status;
	status.dwCheckPoint = 0;
	status.dwControlsAccepted = SERVICE_ACCEPT_STOP | SERVICE_ACCEPT_SHUTDOWN;
	status.dwCurrentState = dwServiceState;
	status.dwServiceSpecificExitCode = 0;
	status.dwServiceType = SERVICE_WIN32_OWN_PROCESS;
	status.dwWaitHint = 0;
	status.dwWin32ExitCode = 0;

	SetServiceStatus(theApp.statusHandle, &status);

	theApp.lastState = dwServiceState;
}


DWORD WINAPI MonitorThread(DWORD dwParameter)
{
	ZopeService *service = (ZopeService *) dwParameter;

	// Monitor the status of Zope
	SetThreadPriority(GetCurrentThread(), THREAD_PRIORITY_BELOW_NORMAL);

	while (1)
	{
		DWORD status = service->QueryStatus();
		if (status != theApp.lastState)
			SetStatus(status);
		if (status == SERVICE_START_PENDING)
			Sleep(1000);
		else
			Sleep(5000);
	}

	// Thread will die when process terminates
}

